DestroyusProApi.minRequireAmountInStack = 5
DestroyusProApi.minRequireAmountInStackForMasWork = 20 -- Mass Mill/Prospecting stack size

function DestroyusProApi.isQualityHerbAllowed(self, quality)
	return quality >= self.strToNumQuality["common"] and 
		   quality <= self.strToNumQuality["uncommon"] 
end

function DestroyusProApi.isItemNotConsumable(self, item_id)
	return not IsConsumableItem(item_id)
end

function DestroyusProApi.isStackAmountEnough(self, amount)
	return amount >= self.minRequireAmountInStack
end

function DestroyusProApi.isStackAmountEnoughForMasWork(self, amount)
	return amount >= self.minRequireAmountInStackForMasWork
end

function DestroyusProApi.isHerb(self, classId, subClassId)
	return self.itemClassIdTradeGoods[classId] and  self.itemSubClassIdHerbs[subClassId]
end

function DestroyusProApi.isOre(self, classId, subClassId, itemId)
	return self.itemClassIdTradeGoods[classId] and self.itemSubClassIdOres[subClassId] and 
	       self.itemClassOreId[itemId]
end

function DestroyusProApi.haveItemRequiredProfLvl(self, item_id, profSkillId, itemsIlvl)
    local profLvLReq = true
	if self.fp:getSkillId() == profSkillId then
		profLvLReq = itemsIlvl[item_id] <= self.fp:getLevel()
	elseif self.sp:getSkillId() == profSkillId then
		profLvLReq = itemsIlvl[item_id] <= self.sp:getLevel()
	else
		profLvLReq = false
	end
	return profLvLReq
end

function DestroyusProApi.isItemMillingable(self, i, j)
	self.cif:setItemPos(i,j)
	return self:isHerb(self.cif:getItemClassId(), self.cif:getItemSubClassId()) and
		   self:isQualityHerbAllowed(self.cif:getQuality()) and
		   self:isItemNotConsumable(self.cif:getItemId()) and 
		   self:isStackAmountEnough(self.cif:getStackSize()) and
		   self:isItemNotSellable(self.cif:isNotSellable()) and
		   self:isNotUniqueItem(self.cif:getItemId(), self.itemsIdNotForMilling) and
		   self:isItemNotlooked(self.cif:isLoked()) and
		   self:haveItemRequiredProfLvl(self.cif:getItemId(), _DPProfessionApi.skilInscription,
                        self.herbsIlvl)
end

function DestroyusProApi.isItemProspectingable(self, i, j)
	self.cif:setItemPos(i,j)
	return self:isOre(self.cif:getItemClassId(), self.cif:getItemSubClassId(), self.cif:getItemId()) and
		   self:isItemNotConsumable(self.cif:getItemId()) and 
		   self:isStackAmountEnough(self.cif:getStackSize()) and 
		   self:isItemNotSellable(self.cif:isNotSellable()) and
		   self:isItemNotlooked(self.cif:isLoked()) and
		   self:haveItemRequiredProfLvl(self.cif:getItemId(), _DPProfessionApi.skillJewelcrafting,
                        self.oresIlvl)
end

---------------------------------------------------------------------------------------------
function DestroyusProApi.countHerbAndOreMass(self, i, j)
	self.cif:setItemPos(i,j)
	if self.countItem[self.cif:getItemId()] then
		self.countItem[self.cif:getItemId()] = self.countItem[self.cif:getItemId()] + self.cif:getStackSize()
	end
end

function DestroyusProApi.clearTableCountItem(self)
	for key in pairs(self.countItem) do
		self.countItem[key] = 0
	end
end

function DestroyusProApi.selectSpellToCast(self, spells)
	for _, spellID in ipairs(spells) do
		if self.spellSet[spellID] then
			local itemID = self.spellSet[spellID]
			if itemID and self.countItem[itemID] and self:isStackAmountEnoughForMasWork(self.countItem[itemID])then
				return spellID, floor(self.countItem[itemID] / self.minRequireAmountInStackForMasWork)
			end
		end 
	end
	return nil
end

function DestroyusProApi.countItemToWork(self)
	for i = 0, NUM_BAG_SLOTS do
		for j = 1, GetContainerNumSlots(i) do 
			self:countHerbAndOreMass(i, j)
		end 
	end
end

DestroyusProApi.countItem = {}
ExtraTabFun.setDefault(DestroyusProApi.countItem, nil)
-- Legion Herb
DestroyusProApi.countItem[_DPContainerItemApi.item["Aethril"]] = 0
DestroyusProApi.countItem[_DPContainerItemApi.item["Dreamleaf"]] = 0
DestroyusProApi.countItem[_DPContainerItemApi.item["Felwort"]] = 0
DestroyusProApi.countItem[_DPContainerItemApi.item["Fjarnskaggl"]] = 0
DestroyusProApi.countItem[_DPContainerItemApi.item["Foxflower"]] = 0
DestroyusProApi.countItem[_DPContainerItemApi.item["Starlight Rose"]] = 0
DestroyusProApi.countItem[_DPContainerItemApi.item["Yseralline Seed"]] = 0
DestroyusProApi.countItem[_DPContainerItemApi.item["Astral Glory"]] = 0
-- Legion Ore
DestroyusProApi.countItem[_DPContainerItemApi.item["Leystone Ore"]] = 0
DestroyusProApi.countItem[_DPContainerItemApi.item["Felslate"]] = 0
DestroyusProApi.countItem[_DPContainerItemApi.item["Empyrium"]] = 0
-- Battle for Azeroth Herb
DestroyusProApi.countItem[_DPContainerItemApi.item["Anchor Weed"]] = 0
DestroyusProApi.countItem[_DPContainerItemApi.item["Riverbud"]] = 0
DestroyusProApi.countItem[_DPContainerItemApi.item["Star Moss"]] = 0
DestroyusProApi.countItem[_DPContainerItemApi.item["Akunda's Bite"]] = 0
DestroyusProApi.countItem[_DPContainerItemApi.item["Winter's Kiss"]] = 0
DestroyusProApi.countItem[_DPContainerItemApi.item["Siren's Pollen"]] = 0
DestroyusProApi.countItem[_DPContainerItemApi.item["Sea Stalk"]] = 0
-- Battle for Azeroth ore
DestroyusProApi.countItem[_DPContainerItemApi.item["Platinum Ore"]] = 0
DestroyusProApi.countItem[_DPContainerItemApi.item["Monelite Ore"]] = 0
DestroyusProApi.countItem[_DPContainerItemApi.item["Storm Silver Ore"]] = 0

DestroyusProApi.spellSet = {}
ExtraTabFun.setDefault(DestroyusProApi.spellSet, nil)
-- Legion Herb
DestroyusProApi.spellSet[_DPSpellApi.spellMassMillAethril] = _DPContainerItemApi.item["Aethril"]
DestroyusProApi.spellSet[_DPSpellApi.spellMassMillDreamleaf] = _DPContainerItemApi.item["Dreamleaf"]
DestroyusProApi.spellSet[_DPSpellApi.spellMassMillFelwort] = _DPContainerItemApi.item["Felwort"]
DestroyusProApi.spellSet[_DPSpellApi.spellMassMillFjarnskaggl] = _DPContainerItemApi.item["Fjarnskaggl"]
DestroyusProApi.spellSet[_DPSpellApi.spellMassMillFoxflower] = _DPContainerItemApi.item["Foxflower"]
DestroyusProApi.spellSet[_DPSpellApi.spellMassMillStarlightRose] = _DPContainerItemApi.item["Starlight Rose"]
DestroyusProApi.spellSet[_DPSpellApi.spellMassMillYserallineSeeds] = _DPContainerItemApi.item["Yseralline Seed"]
DestroyusProApi.spellSet[_DPSpellApi.spellMassMillAstralGlory] = _DPContainerItemApi.item["Astral Glory"]
-- Legion Ore
DestroyusProApi.spellSet[_DPSpellApi.spellMassProspecLeystone] = _DPContainerItemApi.item["Leystone Ore"]
DestroyusProApi.spellSet[_DPSpellApi.spellMassProspectFelslate] = _DPContainerItemApi.item["Felslate"]
DestroyusProApi.spellSet[_DPSpellApi.spellMassProspectEmpyrium] = _DPContainerItemApi.item["Empyrium"]
-- Battle for Azeroth Herb
DestroyusProApi.spellSet[_DPSpellApi.spellMassMillAnchorWeed] = _DPContainerItemApi.item["Anchor Weed"]
DestroyusProApi.spellSet[_DPSpellApi.spellMassMillAkundasBite] = _DPContainerItemApi.item["Akunda's Bite"]
DestroyusProApi.spellSet[_DPSpellApi.spellMassMillRiverbud] = _DPContainerItemApi.item["Riverbud"]
DestroyusProApi.spellSet[_DPSpellApi.spellMassMillSeaStalk] = _DPContainerItemApi.item["Sea Stalk"]
DestroyusProApi.spellSet[_DPSpellApi.spellMassMillSirensPollen] = _DPContainerItemApi.item["Siren's Pollen"]
DestroyusProApi.spellSet[_DPSpellApi.spellMassMillStarMoss] = _DPContainerItemApi.item["Star Moss"]
DestroyusProApi.spellSet[_DPSpellApi.spellMassMillWintersKiss] = _DPContainerItemApi.item["Winter's Kiss"]
-- Battle for Azeroth Ore
DestroyusProApi.spellSet[_DPSpellApi.spellMassProspectPlatinum] = _DPContainerItemApi.item["Platinum Ore"]
DestroyusProApi.spellSet[_DPSpellApi.spellMassProspecStormSilver] = _DPContainerItemApi.item["Storm Silver Ore"]
DestroyusProApi.spellSet[_DPSpellApi.spellMassProspectMonelite] = _DPContainerItemApi.item["Monelite Ore"]