_DPMacroManager = {}
_DPMacroManager.__index = _DPMacroManager
_DPMacroManager.SUCCESS = "SUCCESS"
_DPMacroManager.FAILED = "FAILED"
_DPMacroManager.CREATED = "CREATED"
_DPMacroManager.DELETED = "DELETED"
_DPMacroManager.UPDATE = "UPDATE"

-- Macro Names
-- Old Names
_DPMacroManager.OLDDISENCHANTUS = "Disenchantus"
_DPMacroManager.OLDMILLINGUS = "Millingus"
_DPMacroManager.OLDPROSPECTINGUS = "Prospectingus"
_DPMacroManager.OLDMASSMILLINGUS = "MassMillingus"
_DPMacroManager.OLDMASSPROSPECTINGUS = "MassProspectingus"
-- New Names
_DPMacroManager.DISENCHANTUS = "DesProDisenchant"
_DPMacroManager.MILLINGUS = "DesProMilling"
_DPMacroManager.PROSPECTINGUS = "DesProProspect"
_DPMacroManager.MASSMILLINGUS = "DesProMMLegion"
_DPMacroManager.MASSPROSPECTINGUS = "DesProMPLegion"
_DPMacroManager.MASSMILLINGUSAZEROTH = "DesProMMAzeroth"
_DPMacroManager.MASSPROSPECTINGUSAZEROTH = "DesProMPAzeroth"
_DPMacroManager.DESPRONONE = "DesProNone"

-- Macro Icons
_DPMacroManager.BLUE_BOWL = "237032"
_DPMacroManager.RED_BOWL = "237031"
_DPMacroManager.BLACK_MAGIC_HAND = "645142"
_DPMacroManager.SMALL_ROCK = "133287"
_DPMacroManager.GROUP_OF_SMALL_ROCKS = "134119"
_DPMacroManager.BOX_WITH_RED_DUST = "236995"
_DPMacroManager.BROWN_DUST = "133208"

setmetatable(_DPMacroManager, {
  __call = function (cls)
    return cls.new()
  end,
})

function _DPMacroManager.new(index)
    local self = setmetatable({}, _DPMacroManager)
    self:_init()
    return self
end

function _DPMacroManager._init(self)
    self.isCombatModeOn = false
end

function _DPMacroManager.macroCD(self, action, macro, icon, content, new_name)
    local result = self.SUCCESS
    if self:canCDMacro() then
	    if action == self.CREATED then
		    result = self:createMacro(macro, icon, "/click " .. content)
		elseif action == self.DELETED then
			result = self:deleteMacro(macro)
		elseif action == self.UPDATE then
		    result = self:updateMacro(macro, new_name, icon, "/click " .. content)
		else
		    result = self.FAILED
		end
	else
	    result = self.FAILED
	end
	return result
end

function _DPMacroManager.deleteMacro(self, macro)
    local result = self.SUCCESS
	if self:macroExist(macro) then
		if MacroFrame then
			if MacroFrame:IsVisible() then -- Macro frame is open
				MacroFrame:Hide()
				DeleteMacro(macro)
				MacroFrame:Show()
			else	
				-- Macro frame is closed.
				DeleteMacro(macro)
			end
		else
			DeleteMacro(macro) -- Macro frame do not exist yet
		end
	else
		result = self.FAILED
	end
    return result
end

function _DPMacroManager.createMacro(self, macro, icon, content)
    local result = self.SUCCESS
	if self:macroExist(macro) == false then
		if MacroFrame then
			if MacroFrame:IsVisible() then -- Macro frame is open
				MacroFrame:Hide()
				CreateMacro(macro, icon, content)
				MacroFrame:Show()
			else	
				-- Macro frame is closed.
				CreateMacro(macro, icon, content)
			end
		else
			-- Macro frame do not exist yet.
			CreateMacro(macro, icon, content)
		end
	else
		result = self.FAILED
	end
	return result
end

function _DPMacroManager.updateMacro(self, macro, new_name, icon, content)
    local result = self.SUCCESS
	if self:macroExist(macro) == true then
		EditMacro(macro, new_name, icon, content)
	else
		result = self.FAILED
	end
	return result
end

function _DPMacroManager.setCombatMode(self, combatModeOn)
   self.isCombatModeOn = combatModeOn
end

function _DPMacroManager.canCDMacro(self)
   return self.isCombatModeOn ~= true
end

function _DPMacroManager.macroExist(self, macro)
	return GetMacroInfo(macro) ~= nil
end
