_DPContainerItemApi = {}
_DPContainerItemApi.__index = _DPContainerItemApi

setmetatable(_DPContainerItemApi, {
  __call = function (cls)
    return cls.new()
  end,
})

function _DPContainerItemApi.new()
    local self = setmetatable({}, _DPContainerItemApi)
    self:_init()
    return self
end

function _DPContainerItemApi._init(self)
	self.i = 0
	self.j = 0
end

function _DPContainerItemApi.setItemPos(self, i, j)
	self.i = i
	self.j = j
	self:unpackContainerItemInfo()
	self:unpackItemInfo()
end

function _DPContainerItemApi.unpackContainerItemInfo(self)
	self.texture, self.stack_size, self.is_locked, self.quality, self.is_readable,
		self.is_lootable, self.item_link, self.is_filtered,
		self.is_not_sellable, self.item_id = GetContainerItemInfo(self.i, self.j)
end

function _DPContainerItemApi.unpackItemInfo(self)
	if self:getItemId() == self.unknowItemId then
		self.name, _, _, self.ilvl, self.requir_level, self.class, self.subclass, self.max_stack_size,
		self.equip_slot, _, self.vendor_price, self.item_class_id, 
		self.item_subclass_id, _, _, _, _= nil
	else
		self.name, _, _, self.ilvl, self.requir_level, self.class, self.subclass, self.max_stack_size,
			self.equip_slot, _, self.vendor_price, self.item_class_id, 
			self.item_subclass_id, _, _, _, _= GetItemInfo(self:getItemId())
	end
end

function _DPContainerItemApi.getTexture(self)
	if self.texture == nil then
		return self.unknowTexture
	else
		return self.texture
	end
end

function _DPContainerItemApi.getStackSize(self)
	if self.stack_size == nil then
		return self.unknowStackSize
	else
		return self.stack_size
	end
end

function _DPContainerItemApi.isLoked(self)
	if self.is_locked == nil then
		return self.unknowFalse
	else
		return self.is_locked
	end
end

function _DPContainerItemApi.getQuality(self)
	if self.quality == nil then
		return self.commonQuality
	else
		return self.quality
	end
end

function _DPContainerItemApi.isReadable(self)
	if self.is_readable == nil then
		return self.unknowFalse
	else
		return self.is_readable
	end
end

function _DPContainerItemApi.isLootable(self)
	if self.is_lootable == nil then
		return self.unknowFalse
	else
		return self.is_lootable
	end
end

function _DPContainerItemApi.getLink(self)
	if self.item_link == nil then
		return self.unknowLink
	else
		return self.item_link
	end
end

function _DPContainerItemApi.isFiltered(self)
	if self.is_filtered == nil then
		return self.unknowFalse
	else
		return self.is_filtered
	end
end

function _DPContainerItemApi.isNotSellable(self)
	if self.is_not_sellable == nil then
		return self.unknowFalse
	else
		return self.is_not_sellable
	end
end

function _DPContainerItemApi.getItemId(self)
	if self.item_id == nil then
		return self.unknowItemId
	else
		return self.item_id
	end
end

-- GetItemInfo
function _DPContainerItemApi.getName(self)
	if self.name == nil then
		return self.unknowName
	else
		return self.name
	end
end

function _DPContainerItemApi.getILvl(self)
	local ilvl = select(1, GetDetailedItemLevelInfo(self:getLink()))
	if ilvl == nil then
		ilvl = self.unknowIlvl
	end
	return ilvl
end

function _DPContainerItemApi.getReqLevel(self)
	if self.requir_level == nil then
		return self.unknowReqLevel
	else
		return self.requir_level
	end
end

function _DPContainerItemApi.getClass(self)
	if self.class == nil then
		return self.unknowClass
	else
		return self.class
	end
end

function _DPContainerItemApi.getSubClass(self)
	if self.subclass == nil then
		return self.unknowSubClass
	else
		return self.subclass
	end
end

function _DPContainerItemApi.getMaxStackSize(self)
	if self.max_stack_size == nil then
		return self.unknowMaxStackSize
	end
	return self.max_stack_size
end

function _DPContainerItemApi.getEquipSlot(self)
	if self.equip_slot == nil then
		return self.unknowEquipSlot
	else
		return self.equip_slot
	end
end

function _DPContainerItemApi.getVendorPrice(self)
	if self.vendor_price == nil then
		return self.unknowVendorPrice
	else
		return self.vendor_price
	end
end

function _DPContainerItemApi.getItemClassId(self)
	if self.item_class_id == nil then
		return self.unknowClassId
	else
		return self.item_class_id
	end
end

function _DPContainerItemApi.getItemSubClassId(self)
	if self.item_subclass_id == nil then
		return self.unknowSubClassId
	else
		return self.item_subclass_id
	end
end

-- Static values 
_DPContainerItemApi.unknowTexture = -1
_DPContainerItemApi.unknowStackSize = -1
_DPContainerItemApi.unknowFalse = false
_DPContainerItemApi.commonQuality = 0
_DPContainerItemApi.unknowLink = "?Unknown!"
_DPContainerItemApi.unknowItemId = -1
_DPContainerItemApi.unknowName = "?Unknown!"
_DPContainerItemApi.unknowIlvl = -1
_DPContainerItemApi.unknowReqLevel = 999999999
_DPContainerItemApi.unknowClass = "?Unknown!"
_DPContainerItemApi.unknowClassId = -1
_DPContainerItemApi.unknowSubClassId = -1
_DPContainerItemApi.unknowSubClass = "?Unknown!"
_DPContainerItemApi.unknowMaxStackSize = -1
_DPContainerItemApi.unknowEquipSlot = "?Unknown!"
_DPContainerItemApi.unknowVendorPrice = -1
_DPContainerItemApi.unknowArg = -1

-- Legion Herb
_DPContainerItemApi.item = {}
_DPContainerItemApi.item["Aethril"] = 124101
_DPContainerItemApi.item["Dreamleaf"] = 124102
_DPContainerItemApi.item["Felwort"] = 124106
_DPContainerItemApi.item["Fjarnskaggl"] = 124104
_DPContainerItemApi.item["Foxflower"] = 124103
_DPContainerItemApi.item["Starlight Rose"] = 124105
_DPContainerItemApi.item["Yseralline Seed"] = 128304
_DPContainerItemApi.item["Astral Glory"] = 151565
--- Legio Ore
_DPContainerItemApi.item["Leystone Ore"] = 123918
_DPContainerItemApi.item["Felslate"] = 123919
_DPContainerItemApi.item["Empyrium"] = 151564
-- Battle for Azeroth Herb
_DPContainerItemApi.item["Anchor Weed"] = 152510
_DPContainerItemApi.item["Riverbud"] = 152505
_DPContainerItemApi.item["Star Moss"] = 152506
_DPContainerItemApi.item["Akunda's Bite"] = 152507
_DPContainerItemApi.item["Winter's Kiss"] = 152508
_DPContainerItemApi.item["Siren's Pollen"] = 152509
_DPContainerItemApi.item["Sea Stalk"] = 152511
-- Battle for Azeroth Ore
_DPContainerItemApi.item["Platinum Ore"] = 152513
_DPContainerItemApi.item["Monelite Ore"] = 152512
_DPContainerItemApi.item["Storm Silver Ore"] = 152579