_DPProfessionApi = {}
_DPProfessionApi.__index = _DPProfessionApi

setmetatable(_DPProfessionApi, {
  __call = function (cls, index)
    return cls.new(index)
  end,
})

function _DPProfessionApi.new(index)
    local self = setmetatable({}, _DPProfessionApi)
    self:_init(index)
    return self
end

function _DPProfessionApi._init(self, index)
	self._profIndex = index
end

function _DPProfessionApi._getProfessionInfo(self)
	return GetProfessionInfo(self:_getProfessionIndex(self))
end

function _DPProfessionApi._getProfessionIndex(self)
	local index = select(self._profIndex, GetProfessions())
	if index == nil then
		index = self.indexNotExist
	end
	return index
end

function _DPProfessionApi.getName(self)
	return select(1, self:_getProfessionInfo())
end

function _DPProfessionApi.getIcon(self)
	return select(2, self:_getProfessionInfo())
end

function _DPProfessionApi.getLevel(self)
	return select(3, self:_getProfessionInfo())
end

function _DPProfessionApi.getMaxLevel(self)
	return select(4, self:_getProfessionInfo())
end

function _DPProfessionApi.getNumAbilities(self)
	return select(5, self:_getProfessionInfo())
end

function _DPProfessionApi.getSpellOffset(self)
	return select(6, self:_getProfessionInfo())
end

function _DPProfessionApi.getSkillId(self)
	return select(7, self:_getProfessionInfo())
end

function _DPProfessionApi.getSkillModifier(self)
	return select(8, self:_getProfessionInfo())
end

function _DPProfessionApi.getSpecIndex(self)
	return select(9, self:_getProfessionInfo())
end

function _DPProfessionApi.getSpecOffset(self)
	return select(10, self:_getProfessionInfo())
end

-- Profession skill constant id
_DPProfessionApi.skillEnchanting = 333
_DPProfessionApi.skilInscription = 773
_DPProfessionApi.skillJewelcrafting = 755

-- Profession Index value which do not exist
_DPProfessionApi.indexNotExist = -1


--------------------------------
_DPTradeSkillApi = {}
_DPTradeSkillApi.__index = _DPTradeSkillApi

setmetatable(_DPTradeSkillApi, {
  __call = function (cls)
    return cls.new()
  end,
})

function _DPTradeSkillApi.new()
    local self = setmetatable({}, _DPTradeSkillApi)
    self:_init()
    return self
end

function _DPTradeSkillApi._init(self)
end

function _DPTradeSkillApi.openTradeSkill(self, profession)
    return C_TradeSkillUI.OpenTradeSkill(profession)
end

function _DPTradeSkillApi.closeTradeSkill(self)
    C_TradeSkillUI.CloseTradeSkill()
end

function _DPTradeSkillApi.craftRecipe(self, spellID, amount)
    C_TradeSkillUI.CraftRecipe(spellID, amount)
end

function _DPTradeSkillApi.getRecipeInfo(self, spellID)
  return C_TradeSkillUI.GetRecipeInfo(spellID)
end

function _DPTradeSkillApi.setRecipeRepeatCount(self, spellID, amount)
  C_TradeSkillUI.SetRecipeRepeatCount(spellID, amount)
end

function _DPTradeSkillApi.isRecipeRepeating(self)
	return C_TradeSkillUI.IsRecipeRepeating()
end

_DPTradeSkillApi.spellsMassMillLegion = {}
_DPTradeSkillApi.spellsMassProspectingLegion = {}
_DPTradeSkillApi.spellsMassMillAzeroth = {}
_DPTradeSkillApi.spellsMassProspectingAzeroth = {}

function _DPTradeSkillApi.collectMassWorkSpells(self, spells)
	local collectedSpells = {}
	for _, spellID in ipairs(spells) do
		local receipeInfo = self:getRecipeInfo(spellID)
		if receipeInfo and self:getRecipeInfo(spellID).learned == true then
			_DPInformationPrint("Adding " .. _DPSpellApi(spellID):getSpellName())
			table.insert(collectedSpells, spellID)
		end
	end
	return collectedSpells
end

function _DPTradeSkillApi.collectAllMassSpells(self)
	self.spellsMassMillLegion = self:collectMassWorkSpells(_DPSpellApi.spellsMassMillLegion)
	self.spellsMassMillAzeroth = self:collectMassWorkSpells(_DPSpellApi.spellsMassMillAzeroth)
	self.spellsMassProspectingLegion = self:collectMassWorkSpells(_DPSpellApi.spellsMassProspectingLegion)
	self.spellsMassProspectingAzeroth = self:collectMassWorkSpells(_DPSpellApi.spellsMassProspectingAzeroth)
	_G["DestroyusProSpellsID"]["MillLegion"] = self.spellsMassMillLegion
	_G["DestroyusProSpellsID"]["MillAzeroth"] = self.spellsMassMillAzeroth
	_G["DestroyusProSpellsID"]["ProspectingLegion"] = self.spellsMassProspectingLegion
	_G["DestroyusProSpellsID"]["ProspectingAzeroth"] = self.spellsMassProspectingAzeroth
end

function _DPTradeSkillApi.isMassSpell(self, newSpellID, spells)
	for _, spellID in ipairs(spells) do
		if spellID == newSpellID then
			return true
		end
	end
	return false
end

function _DPTradeSkillApi.isSpellLearned(self, newSpellID, spells)
	for _, spellID in ipairs(spells) do
		if spellID == newSpellID then
			return true
		end
	end
	return false
end

function _DPTradeSkillApi.addNewSpell(self, newSpellID, spells)
	table.insert(spells, newSpellID)
end

function _DPTradeSkillApi.newSpellAppear(self, newSpellID, massSpells, learnedSpells)
	if self:isMassSpell(newSpellID, massSpells) then
		if self:isSpellLearned(newSpellID, learnedSpells) == false then
			self:addNewSpell(newSpellID, learnedSpells)
		end
	end
end
