DestroyusProMainTest = {}
DestroyusProMainTest.__index = DestroyusProMainTest

setmetatable(DestroyusProMainTest, {
  __index = UnitTest, -- this is what makes the inheritance work
  __call = function (cls)
    local self = setmetatable({}, cls)
    UnitTest:_init(self)
    return self
  end
})

function DestroyusProMainTest.setUp(self)
	self.functionProfInfo = GetProfessionInfo
	self.functionGetProf = GetProfessions
	self.bagFunction =  GetContainerItemInfo
	self.GetContainerNumSlots = GetContainerNumSlots
	self.spellBookItemInfo = GetSpellBookItemInfo
	---
    GetContainerItemInfo = function(i, j) return bagClass:returnBagSlot(i, j) end
	GetContainerNumSlots = function(i) return bagClass:returnContainerNumSlot(i) end
	_profApi:setFirstProf(INSCRIPTION_LVL367_INDEX1)
	_profApi:setSecondProf(JEWELCRAFTING_LVL125_INDEX2)
	GetProfessions = function() return  _profApi:getProfessions() end
	GetProfessionInfo = function(index) return _profApi:getProfessionInfo(index) end
	GetSpellBookItemInfo = function(name) return _spellBook:haveSpellInBook(name) end
	_spellBook:setSpellBook(SPELLS_ALL_MILLING_PROSPECTING)
	
	-- Set Disenchant fields set for tests
	DestroyusProMain.changeQualityDisenchanting("uncommon", 2)
	DestroyusProMain.changeQualityDisenchanting("rare", 3)
	DestroyusProMain.changeQualityDisenchanting("epic", 4)
	
	
	-- Enable back all macro buttons
	ClickMeForWorkMilling:Enable()
	ClickMeForWorkDisenchanting:Enable()
	ClickMeForWorkProspecting:Enable()
	ClickMeForWorkMassMillingLegion:Enable()
	ClickMeForWorkMassProspectingLegion:Enable()
end

function DestroyusProMainTest.tearDown(self)
	GetProfessionInfo = self.functionProfInfo
	GetProfessions = self.functionGetProf
	GetContainerItemInfo = self.bagFunction
	GetContainerNumSlots = self.GetContainerNumSlots
	
	-- Set Disenchant fields set for tests
	DestroyusProMain.changeQualityDisenchanting("uncommon", 2)
	DestroyusProMain.changeQualityDisenchanting("rare", -1)
	DestroyusProMain.changeQualityDisenchanting("epic", -1)
	
	-- Enable back all macro buttons
	ClickMeForWorkMilling:Enable()
	ClickMeForWorkDisenchanting:Enable()
	ClickMeForWorkProspecting:Enable()
	ClickMeForWorkMassMillingLegion:Enable()
	ClickMeForWorkMassProspectingLegion:Enable()
end

function DestroyusProMainTest.buildTestForCreateMacro(self, item, created, spell)
	bagClass:setBagSlot(item)
	expected = string.format("/cast %s\n/use %d %d", _DPSpellApi(spell):getSpellName(), item["i"], item["j"])
	result = DestroyusProMain.createMacroUsage(spell, DestroyusProMain.findItemToWork(spell))
	if created == true then
		self:assertEquals(result, expected, string.format("Item=%s, i=%d, j=%d, have different macro body", item["link"], item["i"], item["j"]))	
	else
		self:assertEquals(result, "", string.format("Item=%s, i=%d, j=%d, do not have empty macro body", item["link"], item["i"], item["j"]))
	end
end

function DestroyusProMainTest.test_createMacroInscription(self)
	-- Talandra's Rose found. Profession lvl is ok. 
	self:buildTestForCreateMacro(TALANDRAS_ROSE_COUNT10_3_12, true, _DPSpellApi.spellMilling)			  
	-- Herb found Azshara's Veil. Profession lvl is ok. 
	self:buildTestForCreateMacro(AZSHARAS_VEIL_COUNT10_3_12, true, _DPSpellApi.spellMilling)
end

function DestroyusProMainTest.test_createMacroProspecting(self)
	-- Iron Ore found. Profession lvl is ok. 
	self:buildTestForCreateMacro(IRON_ORE_COUNT10_0_20, true, _DPSpellApi.spellProspecting)						  
	--Obsidium Ore found. Profession lvl is ok.
	self:buildTestForCreateMacro(OBSIDIUM_ORE_COUNT10_0_20, true, _DPSpellApi.spellProspecting)	
end

function DestroyusProMainTest.test_createMacroEnchanting(self)
	_profApi:setFirstProf(ENCHANTING_LVL367_INDEX10)
	_profApi:setSecondProf(UNKNOW_PROFESSIONS)
	-- Item found. Profession lvl is ok. Frigid War Mace ID 36499
	self:buildTestForCreateMacro(FRIGID_WAR_MACE_ONE_HAND_MACE_2_14, true, _DPSpellApi.spellDisenchant)						  
	-- Item found. Low lvl profession. Razorwind Mace ID 55380
	self:buildTestForCreateMacro(RAZORWIND_MACE_ONE_HAND_MACE_1_1, true, _DPSpellApi.spellDisenchant)	
end

-- Looter tests
function DestroyusProMainTest.test_LootItemsAfterMacroRun(self)
	-- Talandra's Rose. Profession lvl is ok. 
    self:buildTestForCreateMacro(TALANDRAS_ROSE_COUNT10_3_12, true, _DPSpellApi.spellMilling)
	-- Spell has been casted. Disable all buttons
	self:assertEquals(DestroyusProMain.itemFoundAndWorkStarted, true,
					  "Item has been found, so work should be able start")
	self:assertEquals(DestroyusProMain.canLootItemsAfterWork, false,
					  "Until this time looting is not allowed")
	DestroyusProMain.spellActions("self", "UNIT_SPELLCAST_START", "player", "castGUID", _DPSpellApi.spellMilling)
	self:assertEquals(DestroyusProMain.buttonCanWorkEnable, false,
					  "All buttons should be disabled")
	
	-- Spellcast sucess. Proced work
	DestroyusProMain.spellActions("self", "UNIT_SPELLCAST_SUCCEEDED", "player", "castGUID", _DPSpellApi.spellMilling)
	self:assertEquals(DestroyusProMain.itemFoundAndWorkStarted, false,
					  "Item has been found, reset value")
	self:assertEquals(DestroyusProMain.canLootItemsAfterWork, true,
					  "Item has been found, can loot from LootBag")
	self:assertEquals(DestroyusProMain.buttonCanWorkEnable, false,
					  "All buttons should be still disabled")
					  
	-- Open loot. Proced work
	DestroyusProMain.lootAfterWork("Self", "LOOT_OPENED", false)
	self:assertEquals(DestroyusProMain.itemFoundAndWorkStarted, false,
					  "Item has been found, work is done")
	self:assertEquals(DestroyusProMain.canLootItemsAfterWork, false,
					  "Item has been found, work is done")
	self:assertEquals(DestroyusProMain.buttonCanWorkEnable, false,
					  "All buttons should be still disabled")
					  

	-- Close loot and rest data. 
	DestroyusProMain.lootAfterWork("Self", "LOOT_CLOSED", false)
	self:assertEquals(DestroyusProMain.itemFoundAndWorkStarted, false,
					  "Item has been found, work is done")
	self:assertEquals(DestroyusProMain.canLootItemsAfterWork, false,
					  "Item has been found, work is done")
	self:assertEquals(DestroyusProMain.buttonCanWorkEnable, true,
					  "All buttons should be still enabled")
end

function DestroyusProMainTest.test_LootItemsAfterMacroRunInterrupted(self)
	-- Talandra's Rose found. Profession lvl is ok.
	self:buildTestForCreateMacro(TALANDRAS_ROSE_COUNT10_3_12, true, _DPSpellApi.spellMilling)					  
	-- Spell has been casted. Disable all buttons
	self:assertEquals(DestroyusProMain.itemFoundAndWorkStarted, true,
					  "Item has been found, so work should be able start")
	self:assertEquals(DestroyusProMain.canLootItemsAfterWork, false,
					  "Until this time looting is not allowed")
	DestroyusProMain.spellActions("self", "UNIT_SPELLCAST_START", "player", "castGUID", _DPSpellApi.spellMilling)
	self:assertEquals(DestroyusProMain.buttonCanWorkEnable, false,
					  "All buttons should be disabled")
	
	-- Spellcast sucess. Proced work
	DestroyusProMain.spellActions("self", "UNIT_SPELLCAST_FAILED", "player", "castGUID", _DPSpellApi.spellMilling)
	self:assertEquals(DestroyusProMain.itemFoundAndWorkStarted, false,
					  "Spell has been interrupted, work is done")
	self:assertEquals(DestroyusProMain.canLootItemsAfterWork, false,
					  "Spell has been interrupted, work is done")
	self:assertEquals(DestroyusProMain.buttonCanWorkEnable, true,
					  "All buttons should be still enabled")
	
    --	Another case for the event UNIT_SPELLCAST_INTERRUPTED
	self:buildTestForCreateMacro(TALANDRAS_ROSE_COUNT10_3_12, true, _DPSpellApi.spellMilling)						  
	-- Spell has been casted. Disable all buttons
	self:assertEquals(DestroyusProMain.itemFoundAndWorkStarted, true,
					  "Item has been found, so work should be able start")
	self:assertEquals(DestroyusProMain.canLootItemsAfterWork, false,
					  "Until this time looting is not allowed")
	DestroyusProMain.spellActions("self", "UNIT_SPELLCAST_START", "player", "castGUID", _DPSpellApi.spellMilling)
	
	self:assertEquals(DestroyusProMain.buttonCanWorkEnable, false,
					  "All buttons should be disabled")
	
	-- Spellcast sucess. Proced work
	DestroyusProMain.spellActions("self", "UNIT_SPELLCAST_INTERRUPTED", "player", "castGUID", _DPSpellApi.spellMilling)
	self:assertEquals(DestroyusProMain.itemFoundAndWorkStarted, false,
					  "Spell has been interrupted, work is done")
	self:assertEquals(DestroyusProMain.canLootItemsAfterWork, false,
					  "Spell has been interrupted, work is done")
	self:assertEquals(DestroyusProMain.buttonCanWorkEnable, true,
					  "All buttons should be still enabled")
end

function DestroyusProMainTest.test_DestroyusProLootAfterIsDPSkill(self)
      -- Check if only specific spells are used during spell events 
      self:assertEquals(DestroyusProMain.DestroyusProLootAfterIsDPSkill(_DPSpellApi.spellMilling), true, "spellMilling for Loot")
      self:assertEquals(DestroyusProMain.DestroyusProLootAfterIsDPSkill(_DPSpellApi.spellDisenchant), true, "spellDisenchant for Loot") 
      self:assertEquals(DestroyusProMain.DestroyusProLootAfterIsDPSkill(_DPSpellApi.spellProspecting), true, "spellProspecting for Loot")
	  self:assertEquals(DestroyusProMain.DestroyusProLootAfterIsDPSkill(_DPSpellApi.unknowSpellName), false, "unknowSpellName for Loot")
end

function DestroyusProMainTest.test_DestroyusProCloseAfterMassSpell(self)
	-- Check if only specific spells are used during spell events 
	self:assertEquals(DestroyusProMain.DestroyusProAfterMassWork(_DPSpellApi.spellMassMillAethril), true, "spellMassMillAethril for close TradeSkill")
	self:assertEquals(DestroyusProMain.DestroyusProAfterMassWork(_DPSpellApi.spellMassMillDreamleaf), true, "spellMassMillDreamleaf for close TradeSkill")
	self:assertEquals(DestroyusProMain.DestroyusProAfterMassWork(_DPSpellApi.spellMassMillFelwort), true, "spellMassMillFelwort for close TradeSkill")
	self:assertEquals(DestroyusProMain.DestroyusProAfterMassWork(_DPSpellApi.spellMassMillFjarnskaggl), true, "spellMassMillFjarnskaggl for close TradeSkill")
	self:assertEquals(DestroyusProMain.DestroyusProAfterMassWork(_DPSpellApi.spellMassMillFoxflower), true, "spellMassMillFoxflower for close TradeSkill")
	self:assertEquals(DestroyusProMain.DestroyusProAfterMassWork(_DPSpellApi.spellMassMillStarlightRose), true, "spellMassMillStarlightRose for close TradeSkill")
	self:assertEquals(DestroyusProMain.DestroyusProAfterMassWork(_DPSpellApi.spellMassMillYserallineSeeds), true, "spellMassMillYserallineSeeds for close TradeSkill")
	self:assertEquals(DestroyusProMain.DestroyusProAfterMassWork(_DPSpellApi.spellMassMillAstralGlory), true, "spellMassMillAstralGlory for close TradeSkill")
	self:assertEquals(DestroyusProMain.DestroyusProAfterMassWork(_DPSpellApi.spellMassProspectFelslate), true, "spellMassProspectFelslate for close TradeSkill")
	self:assertEquals(DestroyusProMain.DestroyusProAfterMassWork(_DPSpellApi.spellMassProspecLeystone), true, "spellMassProspecLeystone for close TradeSkill")
	self:assertEquals(DestroyusProMain.DestroyusProAfterMassWork(_DPSpellApi.spellMassProspectEmpyrium), true, "spellMassProspectEmpyrium for close TradeSkill")
	self:assertEquals(DestroyusProMain.DestroyusProAfterMassWork(_DPSpellApi.unknowSpellName), false, "unknowSpellName for close TradeSkill")
	-- 
	self:assertEquals(DestroyusProMain.DestroyusProAfterMassWork(_DPSpellApi.spellMassMillAnchorWeed), true, "spellMassMillAnchorWeed for close TradeSkill")
	self:assertEquals(DestroyusProMain.DestroyusProAfterMassWork(_DPSpellApi.spellMassMillAkundasBite), true, "spellMassMillAkundasBite for close TradeSkill")
	self:assertEquals(DestroyusProMain.DestroyusProAfterMassWork(_DPSpellApi.spellMassMillRiverbud), true, "spellMassMillRiverbud for close TradeSkill")
	self:assertEquals(DestroyusProMain.DestroyusProAfterMassWork(_DPSpellApi.spellMassMillSeaStalk), true, "spellMassMillSeaStalk for close TradeSkill")
	self:assertEquals(DestroyusProMain.DestroyusProAfterMassWork(_DPSpellApi.spellMassMillSirensPollen), true, "spellMassMillSirensPollen for close TradeSkill")
	self:assertEquals(DestroyusProMain.DestroyusProAfterMassWork(_DPSpellApi.spellMassMillStarMoss), true, "spellMassMillStarMoss for close TradeSkill")
	self:assertEquals(DestroyusProMain.DestroyusProAfterMassWork(_DPSpellApi.spellMassMillWintersKiss), true, "spellMassMillWintersKiss for close TradeSkill")
	self:assertEquals(DestroyusProMain.DestroyusProAfterMassWork(_DPSpellApi.spellMassProspectPlatinum), true, "spellMassProspectPlatinum for close TradeSkill")
	self:assertEquals(DestroyusProMain.DestroyusProAfterMassWork(_DPSpellApi.spellMassProspecStormSilver), true, "spellMassProspecStormSilver for close TradeSkill")
	self:assertEquals(DestroyusProMain.DestroyusProAfterMassWork(_DPSpellApi.spellMassProspectMonelite), true, "spellMassProspectMonelite for close TradeSkill")	  
	self:assertEquals(DestroyusProMain.DestroyusProAfterMassWork(52843), false, "Moonglow Ink should not be avaiable")
end
