DestroyusProSettingsTest = {}
DestroyusProSettingsTest.__index = DestroyusProSettingsTest

setmetatable(DestroyusProSettingsTest, {
	__index = UnitTest, -- this is what makes the inheritance work
	__call = function (cls)
		local self = setmetatable({}, cls)
		UnitTest:_init(self)
		return self
	end
})

function DestroyusProSettingsTest.setUp(self)
	-- Remove all Macros from game
	-- For the test purpose
	DestroyusProMain.macroMgr:macroCD(_DPMacroManager.DELETED,
	_DPMacroManager.DISENCHANTUS, _DPMacroManager.BLACK_MAGIC_HAND,
	DestroyusProMain.buttonNameDisenchanting)
	
	DestroyusProMain.macroMgr:macroCD(_DPMacroManager.DELETED,
	_DPMacroManager.MILLINGUS, _DPMacroManager.RED_BOWL,
	DestroyusProMain.buttonNameMilling)
	
    DestroyusProMain.macroMgr:macroCD(_DPMacroManager.DELETED,
	_DPMacroManager.MASSPROSPECTINGUS, _DPMacroManager.GROUP_OF_SMALL_ROCKS,
	DestroyusProMain.buttonNameMassProspectingLegion)
	
	DestroyusProMain.macroMgr:macroCD(_DPMacroManager.DELETED,
	_DPMacroManager.MASSMILLINGUS, _DPMacroManager.BLUE_BOWL,
	DestroyusProMain.buttonNameMassMillingLegion)

	DestroyusProMain.macroMgr:macroCD(_DPMacroManager.DELETED,
	_DPMacroManager.PROSPECTINGUS, _DPMacroManager.SMALL_ROCK,
	DestroyusProMain.buttonNameProspecting)
	
    DestroyusProMain.macroMgr:macroCD(_DPMacroManager.DELETED, 
    _DPMacroManager.MASSMILLINGUSAZEROTH, _DPMacroManager.BROWN_DUST,
	DestroyusProMain.buttonNameMassMillingAzeroth)
	
    DestroyusProMain.macroMgr:macroCD(_DPMacroManager.DELETED, 
    _DPMacroManager.MASSPROSPECTINGUSAZEROTH, _DPMacroManager.BOX_WITH_RED_DUST,
	DestroyusProMain.buttonNameMassProspectingAzeroth)

	-- Hide frame if is visible
	_DPMainSettingsFrame:Hide()
end

function DestroyusProSettingsTest.tearDown(self)
	_DPMainSettingsFrame:Hide()
	_DPMainSettingsFrame:Show()
	_DPMainSettingsFrame:Hide()
end

function DestroyusProSettingsTest.test_checkboxDisenchantusValue(self)
	DestroyusProMain.changeQualityDisenchanting("uncommon", DestroyusProApi.strToNumQuality["uncommon"])
	DestroyusProMain.changeQualityDisenchanting("rare", DestroyusProApi.strToNumQuality["rare"])
	DestroyusProMain.changeQualityDisenchanting("epic", DestroyusProApi.strToNumQuality["epic"])
	_DPMainSettingsFrame:Show()
	-- Check if Uncommon quality for disenchantus was set
	self:assertEquals(_DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["UncommonQualityS"]:GetChecked(),
	                  true, "Uncommon quality has not been checked")
					  
	-- Check if Rare quality for disenchantus was set
	self:assertEquals(_DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["RareQualityS"]:GetChecked(),
	                  true, "Rare quality has not been checked")
					  
	-- Check if Epic quality for disenchantus was set
	self:assertEquals(_DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["EpicQualityS"]:GetChecked(),
	                  true, "Epic quality has not been checked")
	
    -- Check if disabled is working 	
	_DPMainSettingsFrame:Hide()
	DestroyusProMain.changeQualityDisenchanting("uncommon", DestroyusProApi.unknowNumber)
	DestroyusProMain.changeQualityDisenchanting("rare", DestroyusProApi.unknowNumber)
	DestroyusProMain.changeQualityDisenchanting("epic", DestroyusProApi.unknowNumber)
	_DPMainSettingsFrame:Show()	
	-- Check if Uncommon quality for disenchantus was set
	self:assertEquals(_DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["UncommonQualityS"]:GetChecked(),
	                  false, "Uncommon quality has been checked")
					  
	-- Check if Rare quality for disenchantus was set
	self:assertEquals(_DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["RareQualityS"]:GetChecked(),
	                  false, "Rare quality has been checked")
					  
	-- Check if Epic quality for disenchantus was set
	self:assertEquals(_DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["EpicQualityS"]:GetChecked(),
	                  false, "Epic quality has been checked")
end

function DestroyusProSettingsTest.test_checkboxDisenchantusOnClick(self)
	DestroyusProMain.changeQualityDisenchanting("uncommon", DestroyusProApi.unknowNumber)
	DestroyusProMain.changeQualityDisenchanting("rare", DestroyusProApi.unknowNumber)
	DestroyusProMain.changeQualityDisenchanting("epic", DestroyusProApi.unknowNumber)
	_DPMainSettingsFrame:Show()
	
	-- Trigger CheckButton Uncommon
	_DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["UncommonQualityS"]:SetChecked(true)
	_DPSettingsManagerUI.Uncommon_OnClick(_DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["UncommonQualityS"], "OnClick")
	self:assertEquals(_G["DestroyusProItemQualityValue"]["uncommon"],
	                  DestroyusProApi.strToNumQuality["uncommon"], "Uncommon quality has not change value. Should be 2")

	-- Trigger CheckButton Uncommon
	_DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["UncommonQualityS"]:SetChecked(false)
	_DPSettingsManagerUI.Uncommon_OnClick(_DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["UncommonQualityS"], "OnClick")
	self:assertEquals(_G["DestroyusProItemQualityValue"]["uncommon"],
	                  DestroyusProApi.unknowNumber, "Uncommon quality has not change value. Should be -1")

	-- Trigger CheckButton Rare
	_DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["RareQualityS"]:SetChecked(true)
	_DPSettingsManagerUI.Rare_OnClick(_DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["RareQualityS"], "OnClick")
	self:assertEquals(_G["DestroyusProItemQualityValue"]["rare"],
	                  DestroyusProApi.strToNumQuality["rare"], "Rare quality has not change value. Should be 3")

	-- Trigger CheckButton Rare
	_DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["RareQualityS"]:SetChecked(false)
	_DPSettingsManagerUI.Rare_OnClick(_DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["RareQualityS"], "OnClick")
	self:assertEquals(_G["DestroyusProItemQualityValue"]["rare"],
	                  DestroyusProApi.unknowNumber, "Rare quality has not change value. Should be -1")
					  
	-- Trigger CheckButton Epic
	_DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["EpicQualityS"]:SetChecked(true)
	_DPSettingsManagerUI.Epic_OnClick(_DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["EpicQualityS"], "OnClick")
	self:assertEquals(_G["DestroyusProItemQualityValue"]["epic"],
	                  DestroyusProApi.strToNumQuality["epic"], "Rare quality has not change value. Should be 4")

	-- Trigger CheckButton Epic
	_DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["EpicQualityS"]:SetChecked(false)
	_DPSettingsManagerUI.Epic_OnClick(_DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["EpicQualityS"], "OnClick")
	self:assertEquals(_G["DestroyusProItemQualityValue"]["epic"],
	                  DestroyusProApi.unknowNumber, "Rare quality has not change value. Should be -1")
end

function DestroyusProSettingsTest.test_checkboxPrintoutOnClick(self)
	DestroyusProMain.changeStateForPrintout(false)
	_DPMainSettingsFrame:Show()
	
	-- Check if disabling printout is positive
	_DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["DisablePrintout"]:SetChecked(false)
	_DPSettingsManagerUI.changeStatePrintout_OnClick(
	                    _DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["DisablePrintout"], "OnClick")
	self:assertEquals(_G["DestroyusProPrintOutState"], true, 
	                  "Printout should be avaiable - true")
					  
	-- Check if disabling printout is negative
	_DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["DisablePrintout"]:SetChecked(true)
	_DPSettingsManagerUI.changeStatePrintout_OnClick(
	                    _DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["DisablePrintout"], "OnClick")
	self:assertEquals(_G["DestroyusProPrintOutState"], false, 
	                  "Printout should be disabled - false")
					  
	-- Check if disabling printout is negative. Back to previous version
	_DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["DisablePrintout"]:SetChecked(false)
	_DPSettingsManagerUI.changeStatePrintout_OnClick(
	                    _DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["DisablePrintout"], "OnClick")
	self:assertEquals(_DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["DisablePrintout"]:GetChecked(),
	                  false, "Printout should be true. Checked is negative")
	self:assertEquals(_G["DestroyusProPrintOutState"], true, 
	                  "Printout should be avaiable - true")
end

function DestroyusProSettingsTest.test_checkboxPrintoutValue(self)
	DestroyusProMain.changeStateForPrintout(false)
	_DPMainSettingsFrame:Show()
	-- Check if disabling printout is positive
	self:assertEquals(_DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["DisablePrintout"]:GetChecked(),
	                  true, "Printout should be false. Checked is positive")
					  
	
    -- Check if disabled is working 	
	_DPMainSettingsFrame:Hide()
	DestroyusProMain.changeStateForPrintout(true)
	_DPMainSettingsFrame:Show()	
	
	-- Check if disabling printout is negative
	self:assertEquals(_DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["DisablePrintout"]:GetChecked(),
	                  false, "Printout should be true. Checked is negative")
end

function DestroyusProSettingsTest.test_checkboxMacros(self)
    -- Triger Macro Event
	-- Check if all macro buttons are disabled/not created
	self:assertEquals(_DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["Millingus"]:GetChecked(),
	                  false, "Millingus macro should not exist")
	self:assertEquals(_DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["MassMillingus"]:GetChecked(),
	                  false, "MassMillingus macro should not exist")
	self:assertEquals(_DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["Disenchantus"]:GetChecked(),
	                  false, "Disenchantus macro should not exist")
	self:assertEquals(_DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["Prospectingus"]:GetChecked(),
	                  false, "Prospectingus macro should not exist")
	self:assertEquals(_DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["MassProspectingus"]:GetChecked(),
	                  false, "MassProspectingus macro should not exist")					  
	self:assertEquals(_DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["MassProspectingusAzeroth"]:GetChecked(),
	                  false, "MassProspectingusAzeroth macro should exist")
	self:assertEquals(_DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["MassMillingusAzeroth"]:GetChecked(),
	                  false, "MassMillingusAzeroth macro should exist")
	-- Create all Macros for test
	DestroyusProMain.macroMgr:macroCD(_DPMacroManager.CREATED, 
    _DPMacroManager.MASSPROSPECTINGUS, _DPMacroManager.GROUP_OF_SMALL_ROCKS,
    DestroyusProMain.buttonNameMassProspectingLegion)

    DestroyusProMain.macroMgr:macroCD(_DPMacroManager.CREATED, 
    _DPMacroManager.MASSMILLINGUS, _DPMacroManager.BLUE_BOWL,
    DestroyusProMain.buttonNameMassMillingLegion)

    DestroyusProMain.macroMgr:macroCD(_DPMacroManager.CREATED, 
    _DPMacroManager.PROSPECTINGUS, _DPMacroManager.SMALL_ROCK,
    DestroyusProMain.buttonNameProspecting)

    DestroyusProMain.macroMgr:macroCD(_DPMacroManager.CREATED, 
    _DPMacroManager.MILLINGUS, _DPMacroManager.RED_BOWL,
    DestroyusProMain.buttonNameMilling)

    DestroyusProMain.macroMgr:macroCD(_DPMacroManager.CREATED, 
    _DPMacroManager.DISENCHANTUS, _DPMacroManager.BLACK_MAGIC_HAND,
    DestroyusProMain.buttonNameDisenchanting)
	
    DestroyusProMain.macroMgr:macroCD(_DPMacroManager.CREATED, 
    _DPMacroManager.MASSMILLINGUSAZEROTH, _DPMacroManager.BROWN_DUST,
	DestroyusProMain.buttonNameMassMillingAzeroth)
	
    DestroyusProMain.macroMgr:macroCD(_DPMacroManager.CREATED, 
    _DPMacroManager.MASSPROSPECTINGUSAZEROTH, _DPMacroManager.BOX_WITH_RED_DUST,
    DestroyusProMain.buttonNameMassProspectingAzeroth)

	-- Triger Macro Event
	-- Check if all macro buttons are disabled/not created
	self:assertEquals(_DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["Millingus"]:GetChecked(),
	                  true, "Millingus macro should exist")
	self:assertEquals(_DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["MassMillingus"]:GetChecked(),
	                  true, "MassMillingus macro should exist")
	self:assertEquals(_DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["Disenchantus"]:GetChecked(),
	                  true, "Disenchantus macro should exist")
	self:assertEquals(_DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["Prospectingus"]:GetChecked(),
	                  true, "Prospectingus macro should exist")
	self:assertEquals(_DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["MassProspectingus"]:GetChecked(),
	                  true, "MassProspectingus macro should exist")
	self:assertEquals(_DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["MassProspectingusAzeroth"]:GetChecked(),
	                  true, "MassProspectingusAzeroth macro should exist")
	self:assertEquals(_DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["MassMillingusAzeroth"]:GetChecked(),
	                  true, "MassMillingusAzeroth macro should exist")	  
end

function DestroyusProSettingsTest.test_checkboxMillingusOnClick(self)
	-- Trigger button press
	_DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["Millingus"]:SetChecked(true)
	_DPSettingsManagerUI.Miling_OnClick(
	                    _DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["Millingus"], "OnClick")
	self:assertEquals(DestroyusProMain.macroMgr:macroExist(_DPMacroManager.MILLINGUS), true, 
	                  "Millingus Macro should be created")
	-- Trigger button press
	_DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["Millingus"]:SetChecked(false)
	_DPSettingsManagerUI.Miling_OnClick(
	                    _DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["Millingus"], "OnClick")
	self:assertEquals(DestroyusProMain.macroMgr:macroExist(_DPMacroManager.MILLINGUS), false, 
	                  "Millingus Macro should be deleted")					  
end

function DestroyusProSettingsTest.test_checkboxMassMillingusOnClick(self)
	-- Trigger button press
	_DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["MassMillingus"]:SetChecked(true)
	_DPSettingsManagerUI.MassMilingLegion_OnClick(
	                    _DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["MassMillingus"], "OnClick")
	self:assertEquals(DestroyusProMain.macroMgr:macroExist(_DPMacroManager.MASSMILLINGUS), true, 
	                  "MassMillingus Macro should be created")
	-- Trigger button press
	_DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["MassMillingus"]:SetChecked(false)
	_DPSettingsManagerUI.MassMilingLegion_OnClick(
	                    _DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["MassMillingus"], "OnClick")
	self:assertEquals(DestroyusProMain.macroMgr:macroExist(_DPMacroManager.MASSMILLINGUS), false, 
	                  "MassMillingus Macro should be deleted")					  
end

function DestroyusProSettingsTest.test_checkboxMassProspectingusOnClick(self)
	-- Trigger button press
	_DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["MassProspectingus"]:SetChecked(true)
	_DPSettingsManagerUI.MassProspectingLegion_OnClick(
	                    _DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["MassProspectingus"], "OnClick")
	self:assertEquals(DestroyusProMain.macroMgr:macroExist(_DPMacroManager.MASSPROSPECTINGUS), true, 
	                  "MassProspectingus Macro should be created")
	-- Trigger button press
	_DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["MassProspectingus"]:SetChecked(false)
	_DPSettingsManagerUI.MassProspectingLegion_OnClick(
	                    _DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["MassProspectingus"], "OnClick")
	self:assertEquals(DestroyusProMain.macroMgr:macroExist(_DPMacroManager.MASSPROSPECTINGUS), false, 
	                  "MassProspectingus Macro should be deleted")					  
end

function DestroyusProSettingsTest.test_checkboxDisenchantusOnClick(self)
	-- Trigger button press
	_DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["Disenchantus"]:SetChecked(true)
	_DPSettingsManagerUI.Disenchanting_OnClick(
	                    _DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["Disenchantus"], "OnClick")
	self:assertEquals(DestroyusProMain.macroMgr:macroExist(_DPMacroManager.DISENCHANTUS), true, 
	                  "Disenchantus Macro should be created")
	-- Trigger button press
	_DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["Disenchantus"]:SetChecked(false)
	_DPSettingsManagerUI.Disenchanting_OnClick(
	                    _DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["Disenchantus"], "OnClick")
	self:assertEquals(DestroyusProMain.macroMgr:macroExist(_DPMacroManager.DISENCHANTUS), false, 
	                  "Disenchantus Macro should be deleted")					  
end

function DestroyusProSettingsTest.test_checkboxProspectingusOnClick(self)
	-- Trigger button press
	_DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["Prospectingus"]:SetChecked(true)
	_DPSettingsManagerUI.Prospecting_OnClick(
	                    _DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["Prospectingus"], "OnClick")
	self:assertEquals(DestroyusProMain.macroMgr:macroExist(_DPMacroManager.PROSPECTINGUS), true, 
	                  "Prospectingus Macro should be created")
	-- Trigger button press
	_DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["Prospectingus"]:SetChecked(false)
	_DPSettingsManagerUI.Prospecting_OnClick(
	                    _DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["Prospectingus"], "OnClick")
	self:assertEquals(DestroyusProMain.macroMgr:macroExist(_DPMacroManager.PROSPECTINGUS), false, 
	                  "Prospectingus Macro should be deleted")
	self:assertEquals(_DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["Prospectingus"]:GetChecked(),
	                  false, "Prospectingus macro should exist")					  
end

function DestroyusProSettingsTest.test_checkboxMassProspectingusAzerothOnClick(self)
	-- Trigger button press
	_DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["MassProspectingusAzeroth"]:SetChecked(true)
	_DPSettingsManagerUI.MassProspectingAzeroth_OnClick(
	                    _DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["MassProspectingusAzeroth"], "OnClick")
	self:assertEquals(DestroyusProMain.macroMgr:macroExist(_DPMacroManager.MASSPROSPECTINGUSAZEROTH), true, 
	                  "MassProspectingusAzeroth Macro should be created")
	-- Trigger button press
	_DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["MassProspectingusAzeroth"]:SetChecked(false)
	_DPSettingsManagerUI.MassProspectingAzeroth_OnClick(
	                    _DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["MassProspectingusAzeroth"], "OnClick")
	self:assertEquals(DestroyusProMain.macroMgr:macroExist(_DPMacroManager.MASSPROSPECTINGUSAZEROTH), false, 
	                  "MassProspectingusAzeroth Macro should be deleted")
	self:assertEquals(_DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["MassProspectingusAzeroth"]:GetChecked(),
	                  false, "MassProspectingusAzeroth macro should exist")					  
end

function DestroyusProSettingsTest.test_checkboxMassMillingusAzerothOnClick(self)
	-- Trigger button press
	_DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["MassMillingusAzeroth"]:SetChecked(true)
	_DPSettingsManagerUI.MassMilingAzeroth_OnClick(
	                    _DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["MassMillingusAzeroth"], "OnClick")
	self:assertEquals(DestroyusProMain.macroMgr:macroExist(_DPMacroManager.MASSMILLINGUSAZEROTH), true, 
	                  "MassMillingusAzeroth Macro should be created")
	-- Trigger button press
	_DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["MassMillingusAzeroth"]:SetChecked(false)
	_DPSettingsManagerUI.MassMilingAzeroth_OnClick(
	                    _DPMainSettingsFrame._DPOptionFrame.panel.checkButtons["MassMillingusAzeroth"], "OnClick")
	self:assertEquals(DestroyusProMain.macroMgr:macroExist(_DPMacroManager.MASSMILLINGUSAZEROTH), false, 
	                  "MassMillingusAzeroth Macro should be deleted")					  
end
