ItemsTest = {}
ItemsTest.__index = ItemsTest

setmetatable(ItemsTest, {
  __index = UnitTest, -- this is what makes the inheritance work
  __call = function (cls)
    local self = setmetatable({}, cls)
    UnitTest:_init(self)
    return self
  end
})

-- DISENCHANTING TESTS
function ItemsTest.setUp(self)
	self.functionProfInfo = GetProfessionInfo
	self.functionGetProf = GetProfessions
	self.bagFunction =  GetContainerItemInfo
	self.GetContainerNumSlots = GetContainerNumSlots
	---
	GetProfessions = function() return  _profApi:getProfessions() end
	GetProfessionInfo = function(index) return _profApi:getProfessionInfo(index) end
	GetContainerItemInfo = function(i, j) return bagClass:returnBagSlot(i, j) end
	GetContainerNumSlots = function(i) return bagClass:returnContainerNumSlot(i) end
	self.itemApi = _DPContainerItemApi()
	
	-- Set Disenchant fields set for tests
	DestroyusProMain.changeQualityDisenchanting("uncommon", 2)
	DestroyusProMain.changeQualityDisenchanting("rare", 3)
	DestroyusProMain.changeQualityDisenchanting("epic", 4)
end

function ItemsTest.tearDown(self)
	GetProfessionInfo = self.functionProfInfo
	GetProfessions = self.functionGetProf
	GetContainerItemInfo = self.bagFunction 
	GetContainerNumSlots = self.GetContainerNumSlots
	
	-- Set Disenchant fields set for tests
	DestroyusProMain.changeQualityDisenchanting("uncommon", 2)
	DestroyusProMain.changeQualityDisenchanting("rare", -1)
	DestroyusProMain.changeQualityDisenchanting("epic", -1)
end

function ItemsTest.test_isItem(self)
	-- Sunspring Vest 106719
	bagClass:setBagSlot(SUNSPRING_VEST_1_1)
	self.itemApi:setItemPos(SUNSPRING_VEST_1_1["i"], SUNSPRING_VEST_1_1["j"])
	self:assertEquals(DestroyusPro:isItem(self.itemApi:getItemClassId(),
                 	  self.itemApi:getItemSubClassId(), self.itemApi:getEquipSlot()),
	                  true, "Sunspring Vest should be seen as item!")
					  
	-- Crystallizing Mana 141292
	bagClass:setBagSlot(CRYSTALLIZING_MANA_1_1)
	self.itemApi:setItemPos(CRYSTALLIZING_MANA_1_1["i"], CRYSTALLIZING_MANA_1_1["j"])
	self:assertEquals(DestroyusPro:isItem(self.itemApi:getItemClassId(),
                 	  self.itemApi:getItemSubClassId(), self.itemApi:getEquipSlot()),
	                  true, "Crystallizing Mana should be seen as Artifact Weapon upgrade")
	
	-- Rigid Lapis Lazuli 76502
	bagClass:setBagSlot(RIGID_LAPIS_LAZULI_1_1)
	self.itemApi:setItemPos(RIGID_LAPIS_LAZULI_1_1["i"], RIGID_LAPIS_LAZULI_1_1["j"])
	self:assertEquals(DestroyusPro:isItem(self.itemApi:getItemClassId(),
                 	  self.itemApi:getItemSubClassId(), self.itemApi:getEquipSlot()),
	                  false, "Rigid Lapis Lazuli should not be seen as Artifact Weapon upgrade")
end

function ItemsTest.test_isWeapon(self)
	-- Sunspring Vest 106719
	bagClass:setBagSlot(SUNSPRING_VEST_1_1)
	self.itemApi:setItemPos(SUNSPRING_VEST_1_1["i"], SUNSPRING_VEST_1_1["j"])
	self:assertEquals(DestroyusPro:isWeapon(self.itemApi:getItemClassId(), self.itemApi:getItemSubClassId()),
	                  false, "Sunspring Vest is a Armor!")
	-- Tracker's Musket of the Bandit 36643
	bagClass:setBagSlot(TRACKERS_MUSKET_OF_THE_BANDIT_1_1)
	self.itemApi:setItemPos(TRACKERS_MUSKET_OF_THE_BANDIT_1_1["i"], TRACKERS_MUSKET_OF_THE_BANDIT_1_1["j"])
	self:assertEquals(DestroyusPro:isWeapon(self.itemApi:getItemClassId(), self.itemApi:getItemSubClassId()),
	                  true, "Tracker's Musket of the Bandit should be Weapon!")
	-- Oval Ring of Stamina 36417 
	bagClass:setBagSlot(OVAL_RING_OF_STAMINA_1_1)
	self.itemApi:setItemPos(OVAL_RING_OF_STAMINA_1_1["i"], OVAL_RING_OF_STAMINA_1_1["j"])
	self:assertEquals(DestroyusPro:isWeapon(self.itemApi:getItemClassId(), self.itemApi:getItemSubClassId()),
	                  false, "Oval Ring of Stamina is a Miscellaneous!")

end

function ItemsTest.test_isArtifactWeapon(self)
	-- GRISLY_SOUVENIR_3_2 140413
	bagClass:setBagSlot(GRISLY_SOUVENIR_3_2)
	self.itemApi:setItemPos(GRISLY_SOUVENIR_3_2["i"], GRISLY_SOUVENIR_3_2["j"])
	self:assertEquals(DestroyusPro:isArtifactRelic(self.itemApi:getItemClassId(), self.itemApi:getItemSubClassId()),
	                  true, "Grisly Souvenir is a Artifact Weapon upgrade!")
					  
	-- Nightmare Tear 140413
	bagClass:setBagSlot(NIGHTMARE_TEAR_1_1)
	self.itemApi:setItemPos(NIGHTMARE_TEAR_1_1["i"], NIGHTMARE_TEAR_1_1["j"])
	self:assertEquals(DestroyusPro:isArtifactRelic(self.itemApi:getItemClassId(), self.itemApi:getItemSubClassId()),
	                  false, "Nightmare Tear is not a Artifact Weapon upgrade!")
end

function ItemsTest.test_isArmor(self)
	-- Sunspring Vest 106719
	bagClass:setBagSlot(SUNSPRING_VEST_1_1)
	self.itemApi:setItemPos(SUNSPRING_VEST_1_1["i"], SUNSPRING_VEST_1_1["j"])
	self:assertEquals(DestroyusPro:isArmor(self.itemApi:getItemClassId(), self.itemApi:getItemSubClassId()),
	                  true, "Sunspring Vest should be armor!")
	-- Tracker's Musket of the Bandit 36643 
	bagClass:setBagSlot(TRACKERS_MUSKET_OF_THE_BANDIT_1_1)
	self.itemApi:setItemPos(TRACKERS_MUSKET_OF_THE_BANDIT_1_1["i"], TRACKERS_MUSKET_OF_THE_BANDIT_1_1["j"])
	self:assertEquals(DestroyusPro:isArmor(self.itemApi:getItemClassId(), self.itemApi:getItemSubClassId()),
	                  false, "Tracker's Musket of the Bandit is Weapon!") 
	-- Oval Ring of Stamina 36417 
	bagClass:setBagSlot(OVAL_RING_OF_STAMINA_1_1)
	self.itemApi:setItemPos(OVAL_RING_OF_STAMINA_1_1["i"], OVAL_RING_OF_STAMINA_1_1["j"])
	self:assertEquals(DestroyusPro:isArmor(self.itemApi:getItemClassId(), self.itemApi:getItemSubClassId()),
	                  false, "Oval Ring of Stamina is a Miscellaneous!") 
	
end 

function ItemsTest.test_isMiscellaneous(self)
	-- Sunspring Vest 106719
	bagClass:setBagSlot(SUNSPRING_VEST_1_1)
	self.itemApi:setItemPos(SUNSPRING_VEST_1_1["i"], SUNSPRING_VEST_1_1["j"])
	self:assertEquals(DestroyusPro:isMiscellaneous(self.itemApi:getItemClassId(), self.itemApi:getItemSubClassId(),
	                  self.itemApi:getEquipSlot()),
	                  false, "Sunspring Vest is armor!")
	-- Tracker's Musket of the Bandit 36643 
	bagClass:setBagSlot(TRACKERS_MUSKET_OF_THE_BANDIT_1_1)
	self.itemApi:setItemPos(TRACKERS_MUSKET_OF_THE_BANDIT_1_1["i"], TRACKERS_MUSKET_OF_THE_BANDIT_1_1["j"])
	self:assertEquals(DestroyusPro:isMiscellaneous(self.itemApi:getItemClassId(), self.itemApi:getItemSubClassId(),
	                 self.itemApi:getEquipSlot()),
	                 false, "Tracker's Musket of the Bandit is Weapon!")
	-- Oval Ring of Stamina 36417 
	bagClass:setBagSlot(OVAL_RING_OF_STAMINA_1_1)
	self.itemApi:setItemPos(OVAL_RING_OF_STAMINA_1_1["i"], OVAL_RING_OF_STAMINA_1_1["j"])
	self:assertEquals(DestroyusPro:isMiscellaneous(self.itemApi:getItemClassId(), self.itemApi:getItemSubClassId(),
                      self.itemApi:getEquipSlot()),
	                  true, "Oval Ring of Stamina should be a Miscellaneous!")
end 

function ItemsTest.test_getItemProffLvlToDisArmor(self)
	-- Sunspring Vest 106719
	bagClass:setBagSlot(SUNSPRING_VEST_1_1)
	self.itemApi:setItemPos(SUNSPRING_VEST_1_1["i"], SUNSPRING_VEST_1_1["j"])
	self:assertEquals(DestroyusPro:getItemProffLvlToDis(self.itemApi:getILvl(), self.itemApi:getQuality()),
	                 1, "Prof lvl is different than 1!")
	self:assertEquals(DestroyusPro:getItemProffLvlToDis(nil, 3),
	                  DestroyusPro.numberNotExist, "Value does not exist! ")
	-- Holy Shroud 2721 rare
	bagClass:setBagSlot(HOLY_SHROUD_RARE_1_1)
	self.itemApi:setItemPos(HOLY_SHROUD_RARE_1_1["i"], HOLY_SHROUD_RARE_1_1["j"])
	self:assertEquals(DestroyusPro:getItemProffLvlToDis(self.itemApi:getILvl(), self.itemApi:getQuality()),
	                 1, "Prof lvl is different than 1!")
	-- Belt of the Blood Nova 50015 epic
	bagClass:setBagSlot(BELT_OF_THE_BLOOD_NOVA_EPIC_1_1)
	self.itemApi:setItemPos(BELT_OF_THE_BLOOD_NOVA_EPIC_1_1["i"], BELT_OF_THE_BLOOD_NOVA_EPIC_1_1["j"])
	self:assertEquals(DestroyusPro:getItemProffLvlToDis(self.itemApi:getILvl(), self.itemApi:getQuality()),
	                  1, "Prof lvl is different than 1!") 	
end

function ItemsTest.test_getItemProffLvlToDisWeapon(self)
	-- Tracker's Musket of the Bandit 36643 
	bagClass:setBagSlot(TRACKERS_MUSKET_OF_THE_BANDIT_1_1)
	self.itemApi:setItemPos(TRACKERS_MUSKET_OF_THE_BANDIT_1_1["i"], TRACKERS_MUSKET_OF_THE_BANDIT_1_1["j"])
	self:assertEquals(DestroyusPro:getItemProffLvlToDis(self.itemApi:getILvl(), self.itemApi:getQuality()),
	                  1, "Value of weapon is different tan 1!")
	self:assertEquals(DestroyusPro:getItemProffLvlToDis(-1, 3),
	                  DestroyusPro.numberNotExist, "Number should not exist. Case 1")
	self:assertEquals(DestroyusPro:getItemProffLvlToDis(-1, 4),
	                  DestroyusPro.numberNotExist, "Number should not exist. Case 2")
	self:assertEquals(DestroyusPro:getItemProffLvlToDis(-1, 2),
	                  DestroyusPro.numberNotExist, "Number should not exist. Case 3")
	self:assertEquals(DestroyusPro:getItemProffLvlToDis(1, -1), 
					  DestroyusPro.numberNotExist, "Number should not exist. Case 4")
end

function ItemsTest.test_getItemProffLvlToDisRing(self)
	-- Oval Ring of Stamina 36417 
	bagClass:setBagSlot(OVAL_RING_OF_STAMINA_1_1)
	self.itemApi:setItemPos(OVAL_RING_OF_STAMINA_1_1["i"], OVAL_RING_OF_STAMINA_1_1["j"])
	self:assertEquals(DestroyusPro:getItemProffLvlToDis(
                      self.itemApi:getILvl(), self.itemApi:getQuality()),
	                  1, "Value is different than 1!")
end

function ItemsTest.test_isOnlyOneMaxStack(self)
	-- Tracker's Musket of the Bandit 36643 
	bagClass:setBagSlot(TRACKERS_MUSKET_OF_THE_BANDIT_1_1)
	self.itemApi:setItemPos(TRACKERS_MUSKET_OF_THE_BANDIT_1_1["i"], TRACKERS_MUSKET_OF_THE_BANDIT_1_1["j"])
	self:assertEquals(DestroyusPro:isOnlyOneMaxStack(
	                  self.itemApi:getMaxStackSize()), true,
					  "Item do not have one stack")
					  
	-- Gromsblood 8846
	bagClass:setBagSlot(GROMSBLOOD_COUNT10_1_1)
	self.itemApi:setItemPos(GROMSBLOOD_COUNT10_1_1["i"], GROMSBLOOD_COUNT10_1_1["j"])
	self:assertEquals(DestroyusPro:isOnlyOneMaxStack(
	                 self.itemApi:getMaxStackSize()), false,
					 "Item should not be only one stack!")
end

function ItemsTest.test_isQualityAllowed(self)
	-- Tracker's Musket of the Bandit 36643 
	bagClass:setBagSlot(TRACKERS_MUSKET_OF_THE_BANDIT_1_1)
	self.itemApi:setItemPos(TRACKERS_MUSKET_OF_THE_BANDIT_1_1["i"], TRACKERS_MUSKET_OF_THE_BANDIT_1_1["j"])
	self:assertEquals(DestroyusPro:isQualityAllowed(
	                  self.itemApi:getQuality()), true,
					  "Quality should be allowed!")
end

function ItemsTest.test_isItemDisenchantable(self) 
	_profApi:setFirstProf(ENCHANTING_LVL421_INDEX3)
	_profApi:setSecondProf(JEWELCRAFTING_LVL125_INDEX2)
	
    -- Sunspring Vest 106719
	bagClass:setBagSlot(SUNSPRING_VEST_1_1)
	self:assertEquals(DestroyusPro:isItemDisenchantable(
					  SUNSPRING_VEST_1_1["i"],
	                  SUNSPRING_VEST_1_1["j"]),
	                  true, "Sunspring Vest 106719")
					  
	-- Scorching Cloak 134634
	bagClass:setBagSlot(SCORCHING_CLOAK_1_1)
	self:assertEquals(DestroyusPro:isItemDisenchantable(
					  SCORCHING_CLOAK_1_1["i"],
	                  SCORCHING_CLOAK_1_1["j"]),
	                  true, "Scorching Cloak 134634")
					  
	-- Monastic Ring of Stoicism  88638  UniqueId
	bagClass:setBagSlot(MONASTIC_RING_OF_STOICISM_1_1)
	self:assertEquals(DestroyusPro:isItemDisenchantable(
					  MONASTIC_RING_OF_STOICISM_1_1["i"],
	                  MONASTIC_RING_OF_STOICISM_1_1["j"]),
	                  false, "Monastic Ring of Stoicism  88638  UniqueId")
					  
	-- Necropolis Breastplate 36397
	bagClass:setBagSlot(NECROPOLIS_BREASTPLATE_1_1)
	self:assertEquals(DestroyusPro:isItemDisenchantable(
					  NECROPOLIS_BREASTPLATE_1_1["i"],
	                  NECROPOLIS_BREASTPLATE_1_1["j"]),
	                  true, "Necropolis Breastplate 36397")
					  
	-- Belt of the Blood Nova 50015 Quality epic 
	bagClass:setBagSlot(BELT_OF_THE_BLOOD_NOVA_EPIC_1_1)
	self:assertEquals(DestroyusPro:isItemDisenchantable(
					  BELT_OF_THE_BLOOD_NOVA_EPIC_1_1["i"],
	                  BELT_OF_THE_BLOOD_NOVA_EPIC_1_1["j"]),
	                  true, "Belt of the Blood Nova 50015 Quality epic") 
					  
	-- Jacinth Circle 11969
	bagClass:setBagSlot(JACINTH_CIRCLE_1_1)
	self:assertEquals(DestroyusPro:isItemDisenchantable(
					  JACINTH_CIRCLE_1_1["i"],
	                  JACINTH_CIRCLE_1_1["j"]),
	                  true, "Jacinth Circle 11969")
					  
	-- Hypnotist's Watch 25786 Trinket 225 ILVL
	bagClass:setBagSlot(HYPNOTISTS_WATCH_TRINKET_1_1)
	self:assertEquals(DestroyusPro:isItemDisenchantable(
					  HYPNOTISTS_WATCH_TRINKET_1_1["i"],
	                  HYPNOTISTS_WATCH_TRINKET_1_1["j"]),
					  true, "Hypnotist's Watch 25786 Trinket 225 ILVL")
end

function ItemsTest.test_isItemDisenchantableLowLvlSkill(self) 
	_profApi:setFirstProf(JEWELCRAFTING_LVL125_INDEX2)
	_profApi:setSecondProf(ENCHANTING_LVL1_INDEX4)
	
	-- Sunspring Vest 106719
	bagClass:setBagSlot(SUNSPRING_VEST_1_1)
	self:assertEquals(DestroyusPro:isItemDisenchantable(
					  SUNSPRING_VEST_1_1["i"],
	                  SUNSPRING_VEST_1_1["j"]),
	                  true, "Sunspring Vest 106719")
					  
	-- Scorching Cloak 134634
	bagClass:setBagSlot(SCORCHING_CLOAK_1_1)
	self:assertEquals(DestroyusPro:isItemDisenchantable(
					  SCORCHING_CLOAK_1_1["i"],
	                  SCORCHING_CLOAK_1_1["j"]),
	                  true, "Scorching Cloak 134634")
					  
	-- Monastic Ring of Stoicism  88638  UniqueId
	bagClass:setBagSlot(MONASTIC_RING_OF_STOICISM_1_1)
	self:assertEquals(DestroyusPro:isItemDisenchantable(
					  MONASTIC_RING_OF_STOICISM_1_1["i"],
	                  MONASTIC_RING_OF_STOICISM_1_1["j"]),
	                  false, "Monastic Ring of Stoicism  88638  UniqueId")

	-- Necropolis Breastplate 36397
	bagClass:setBagSlot(NECROPOLIS_BREASTPLATE_1_1)
	self:assertEquals(DestroyusPro:isItemDisenchantable(
					  NECROPOLIS_BREASTPLATE_1_1["i"],
	                  NECROPOLIS_BREASTPLATE_1_1["j"]),
	                  true, "Necropolis Breastplate 36397")
					  
	-- Belt of the Blood Nova 50015 Quality epic
	bagClass:setBagSlot(BELT_OF_THE_BLOOD_NOVA_EPIC_1_1)
	self:assertEquals(DestroyusPro:isItemDisenchantable(
					  BELT_OF_THE_BLOOD_NOVA_EPIC_1_1["i"],
	                  BELT_OF_THE_BLOOD_NOVA_EPIC_1_1["j"]),
	                  true, "Belt of the Blood Nova 50015 Quality epic")
					  
	-- Jacinth Circle 11969
	bagClass:setBagSlot(JACINTH_CIRCLE_1_1)
	self:assertEquals(DestroyusPro:isItemDisenchantable(
				      JACINTH_CIRCLE_1_1["i"], 
					  JACINTH_CIRCLE_1_1["j"]),
	                  true, "Jacinth Circle 11969")
					  
	-- Vindicator's Iron Legguards 26033 1 lvl disenchanting and armor
	bagClass:setBagSlot(VINDICATORS_IRON_LEGGUARDS_1_1)
	self:assertEquals(DestroyusPro:isItemDisenchantable(
					  VINDICATORS_IRON_LEGGUARDS_1_1["i"],
	                  VINDICATORS_IRON_LEGGUARDS_1_1["j"]),
	                  true, "Vindicator's Iron Legguards 26033 1 lvl disenchanting and armor")

	-- Crystallizing Mana 141292 1 lvl disenchanting
	bagClass:setBagSlot(CRYSTALLIZING_MANA_1_1)
	self:assertEquals(DestroyusPro:isItemDisenchantable(
					  CRYSTALLIZING_MANA_1_1["i"],
	                  CRYSTALLIZING_MANA_1_1["j"]),
	                  true, "Crystallizing Mana 141292 1 lvl disenchanting")
					  
	-- Grisly Souvenir 140413 1 lvl disenchanting
	bagClass:setBagSlot(GRISLY_SOUVENIR_3_2)
	self:assertEquals(DestroyusPro:isItemDisenchantable(
					  GRISLY_SOUVENIR_3_2["i"],
	                  GRISLY_SOUVENIR_3_2["j"]),
	                  true, "Grisly Souvenir 140413 1 lvl disenchanting rare")

	-- Felsoul Vortex 147112 1 lvl disenchanting
	bagClass:setBagSlot(FELSOUL_VORTEX_1_1)
	self:assertEquals(DestroyusPro:isItemDisenchantable(
					  FELSOUL_VORTEX_1_1["i"],
	                  FELSOUL_VORTEX_1_1["j"]),
	                  true, "Felsoul Vortex 147112 1 lvl disenchanting epic")
end

function ItemsTest.test_isItemDisenchantableTranslateItems(self)
	_profApi:setFirstProf(JEWELCRAFTING_LVL125_INDEX2)
	_profApi:setSecondProf(ENCHANTING_LVL1_INDEX4)
	
	-- Emergency Hatchet 69223 One Hand Axe
	bagClass:setBagSlot(EMERGENCY_HATCHET_ONE_HAND_AXE_1_1)
	self:assertEquals(DestroyusPro:isItemDisenchantable(
					  EMERGENCY_HATCHET_ONE_HAND_AXE_1_1["i"],
	                  EMERGENCY_HATCHET_ONE_HAND_AXE_1_1["j"]),
	                  true, "Emergency Hatchet 69223 One Hand Axe")
					  
	-- Lupine Axe 1220 Two Hand Axe
	bagClass:setBagSlot(LUPIN_AXE_TWO_HAND_AXE_1_1)
	self:assertEquals(DestroyusPro:isItemDisenchantable(
					  LUPIN_AXE_TWO_HAND_AXE_1_1["i"],
	                  LUPIN_AXE_TWO_HAND_AXE_1_1["j"]),
	                  true, "Lupine Axe 1220 Two Hand Axe")
					  
	-- Shipwreck Bow 52646 Bow
	bagClass:setBagSlot(SHIPWRECK_BOW_1_1)
	self:assertEquals(DestroyusPro:isItemDisenchantable(
					  SHIPWRECK_BOW_1_1["i"],
	                  SHIPWRECK_BOW_1_1["j"]),
	                  true, "Shipwreck Bow 52646 Bow")
					  
	-- Skorn's Rifle 3079 Gun
	bagClass:setBagSlot(SKORNS_RIFLE_GUN_1_1)	
	self:assertEquals(DestroyusPro:isItemDisenchantable(
				      SKORNS_RIFLE_GUN_1_1["i"],
	                  SKORNS_RIFLE_GUN_1_1["j"]),
	                  true, "Skorn's Rifle 3079 Gun")
					  
	--Gnoll Punisher 1214 One Hand Mace
	bagClass:setBagSlot(GNOLL_PUNISHER_ONE_HAND_MACE_1_1)	
	self:assertEquals(DestroyusPro:isItemDisenchantable(
					  GNOLL_PUNISHER_ONE_HAND_MACE_1_1["i"],
	                  GNOLL_PUNISHER_ONE_HAND_MACE_1_1["j"]),
					  true, "Gnoll Punisher 1214 One Hand Mace")
					  
	-- Codex Breaker 62878 Two Hand Mace
	bagClass:setBagSlot(CODEX_BREAKER_TWO_HAND_MACE_1_1)
	self:assertEquals(DestroyusPro:isItemDisenchantable(
					  CODEX_BREAKER_TWO_HAND_MACE_1_1["i"],
	                  CODEX_BREAKER_TWO_HAND_MACE_1_1["j"]),
					  true, "Codex Breaker 62878 Two Hand Mace")
					  
	-- Stunted Tree Root 57214 Polearms
	bagClass:setBagSlot(STUNTED_TREE_ROOT_POLEARMS_1_1)
	self:assertEquals(DestroyusPro:isItemDisenchantable(
				      STUNTED_TREE_ROOT_POLEARMS_1_1["i"],
	                  STUNTED_TREE_ROOT_POLEARMS_1_1["j"]),
					  true, "Stunted Tree Root 57214 Polearms")
					  
	-- Sunwell Blade 22783 One Hand Sword
	bagClass:setBagSlot(SUNWELL_BLADE_ONE_HAND_SWORD_1_1)
	self:assertEquals(DestroyusPro:isItemDisenchantable(
			          SUNWELL_BLADE_ONE_HAND_SWORD_1_1["i"],
	                  SUNWELL_BLADE_ONE_HAND_SWORD_1_1["j"]),
					  true, "Sunwell Blade 22783 One Hand Sword")
					  
	-- Brashclaw's Skewer 2204 Two hand Sword
	bagClass:setBagSlot(BRASHCLAWS_SKEWER_TWO_HAND_SWORD_1_1)
	self:assertEquals(DestroyusPro:isItemDisenchantable(
		              BRASHCLAWS_SKEWER_TWO_HAND_SWORD_1_1["i"],
	                  BRASHCLAWS_SKEWER_TWO_HAND_SWORD_1_1["j"]),
					  true, "Brashclaw's Skewer 2204 Two hand Sword")
					  
	-- Gnarled Hermit's Staff 1539 Staff
	bagClass:setBagSlot(GNARLED_HERMITS_STAFF_1_1)
	self:assertEquals(DestroyusPro:isItemDisenchantable(
	                  GNARLED_HERMITS_STAFF_1_1["i"],
	                  GNARLED_HERMITS_STAFF_1_1["j"]),
			          true, "Gnarled Hermit's Staff 1539 Staff")
					  
	-- Filthy Paw  58978 Fist Weapon
	bagClass:setBagSlot(FILTHY_PAW_FIST_WEAPON_1_1)
	self:assertEquals(DestroyusPro:isItemDisenchantable(
			          FILTHY_PAW_FIST_WEAPON_1_1["i"],
	                  FILTHY_PAW_FIST_WEAPON_1_1["j"]),
	                  true, "Filthy Paw  58978 Fist Weapon")
					  
	-- Venom Web Fang  899 Dagger
	bagClass:setBagSlot(VENOM_WEB_FANG_DAGGER_1_1)
	self:assertEquals(DestroyusPro:isItemDisenchantable(
		              VENOM_WEB_FANG_DAGGER_1_1["i"],
	                  VENOM_WEB_FANG_DAGGER_1_1["j"]),
	                  true, "Venom Web Fang  899 Dagger")
					  
	-- Shrewd Crossbow  59582 Xbow
	bagClass:setBagSlot(SHREWD_CROSSBOW_1_1)
	self:assertEquals(DestroyusPro:isItemDisenchantable(
	                  SHREWD_CROSSBOW_1_1["i"],
	                  SHREWD_CROSSBOW_1_1["j"]),
	                  true, "Shrewd Crossbow  59582 Xbow")
					  
	-- Brazie's Plant Light 65689 Wand
	bagClass:setBagSlot(BRAZIES_PLANT_LIGHT_WAND_1_1)
	self:assertEquals(DestroyusPro:isItemDisenchantable(
			          BRAZIES_PLANT_LIGHT_WAND_1_1["i"],
					  BRAZIES_PLANT_LIGHT_WAND_1_1["j"]),	
	                  true, "Brazie's Plant Light 65689 Wand")

	-- Glaive of the Fallen 141604 Warglaives
	bagClass:setBagSlot(GLAIVE_OF_THE_FALLEN_WARGLAIVES_1_1)
	self:assertEquals(DestroyusPro:isItemDisenchantable(
		              GLAIVE_OF_THE_FALLEN_WARGLAIVES_1_1["i"],
					  GLAIVE_OF_THE_FALLEN_WARGLAIVES_1_1["j"]),	
	                  true, "Glaive of the Fallen 141604 Warglaives")
end

function ItemsTest.test_isItemDisenchantableTranslateArmor(self)
	_profApi:setFirstProf(JEWELCRAFTING_LVL125_INDEX2)
	_profApi:setSecondProf(ENCHANTING_LVL1_INDEX4)
	
	-- Blue Overalls 6263 Cloth Armor
	bagClass:setBagSlot(BLUE_OVERALLS_CLOTH_ARMOR_1_1)
	self:assertEquals(DestroyusPro:isItemDisenchantable(
		              BLUE_OVERALLS_CLOTH_ARMOR_1_1["i"],
					  BLUE_OVERALLS_CLOTH_ARMOR_1_1["j"]),
					  true, "Blue Overalls 6263 Cloth Armor")
					  
	-- Dark Leather Tunic 2317 Leather Armor
	bagClass:setBagSlot(DARK_LEARHER_TUNIC_LEATHER_ARMOR_1_1)
	self:assertEquals(DestroyusPro:isItemDisenchantable(
	                  DARK_LEARHER_TUNIC_LEATHER_ARMOR_1_1["i"],
					  DARK_LEARHER_TUNIC_LEATHER_ARMOR_1_1["j"]),
					  true, "Dark Leather Tunic 2317 Leather Armor")
					  
	-- Ironforge Breastplate 6731 Mail Armor
	bagClass:setBagSlot(IRONFORGE_BREASTPLATE_MAIL_ARMOR)
	self:assertEquals(DestroyusPro:isItemDisenchantable(
					  IRONFORGE_BREASTPLATE_MAIL_ARMOR["i"],
					  IRONFORGE_BREASTPLATE_MAIL_ARMOR["j"]),
					  true, "Ironforge Breastplate 6731 Mail Armor")
					  
	-- Stabilized Incendicite Legguards 59066 Plate
	bagClass:setBagSlot(STABILIZED_INCENDICITE_LEGGUARDS_PLATE_ARMOR_1_1)
	self:assertEquals(DestroyusPro:isItemDisenchantable(
					  STABILIZED_INCENDICITE_LEGGUARDS_PLATE_ARMOR_1_1["i"],
					  STABILIZED_INCENDICITE_LEGGUARDS_PLATE_ARMOR_1_1["j"]),
	                  true, "Stabilized Incendicite Legguards 59066 Plate")
					  
	-- Wheel of the Lost Hope 24334 Shield
	bagClass:setBagSlot(WHEEL_OF_THE_LOST_HOPE_SHIELD_1_1)
	self:assertEquals(DestroyusPro:isItemDisenchantable(
					  WHEEL_OF_THE_LOST_HOPE_SHIELD_1_1["i"],
					  WHEEL_OF_THE_LOST_HOPE_SHIELD_1_1["j"]),
	                  true, "Wheel of the Lost Hope 24334 Shield")
					  
	-- Greenweave Cloak 9770 back
	bagClass:setBagSlot(GREENWAVE_CLOAK_BACK_1_1)
	self:assertEquals(DestroyusPro:isItemDisenchantable(
					  GREENWAVE_CLOAK_BACK_1_1["i"],
					  GREENWAVE_CLOAK_BACK_1_1["j"]),
	                  true, "Greenweave Cloak 9770 back")
end

function ItemsTest.test_isItemDisenchantableTranslateMiscellaneous(self)
	_profApi:setFirstProf(JEWELCRAFTING_LVL125_INDEX2)
	_profApi:setSecondProf(ENCHANTING_LVL1_INDEX4)
	
	-- Malachite Pendant 25438 Neck
	bagClass:setBagSlot(MALACHITE_PENDANT_NECK_1_1)
	self:assertEquals(DestroyusPro:isItemDisenchantable(
				      MALACHITE_PENDANT_NECK_1_1["i"],
					  MALACHITE_PENDANT_NECK_1_1["j"]),
	                  true, "Malachite Pendant 25438 Neck")
					  
	-- Windrunner Band 62892 Ring
	bagClass:setBagSlot(WINDRUNNER_BAND_RING_1_1)
	self:assertEquals(DestroyusPro:isItemDisenchantable(
				      WINDRUNNER_BAND_RING_1_1["i"],
					  WINDRUNNER_BAND_RING_1_1["j"]),
	                  true, "Windrunner Band 62892 Ring")
					  
	-- Hypnotist's Watch 25786 Trinket 225 ILVL
	bagClass:setBagSlot(HYPNOTISTS_WATCH_TRINKET_1_1)
	self:assertEquals(DestroyusPro:isItemDisenchantable(
					  HYPNOTISTS_WATCH_TRINKET_1_1["i"],
					  HYPNOTISTS_WATCH_TRINKET_1_1["j"]),
					  true, "Hypnotist's Watch 25786 Trinket 225 ILVL")
					  
	-- Glowing Murloc Eye 52652 Off Hand
	bagClass:setBagSlot(GLOWING_MURLOCK_EYE_OFF_HAND_1_1)
	self:assertEquals(DestroyusPro:isItemDisenchantable(
			          GLOWING_MURLOCK_EYE_OFF_HAND_1_1["i"],
					  GLOWING_MURLOCK_EYE_OFF_HAND_1_1["j"]),
					  true, "Glowing Murloc Eye 52652 Off Hand")
end

function ItemsTest.test_isFoundBug(self)
	-- Fixed ugly bug which exist from the beginning of the addon. 
	-- Current fault appeared when Enchanting profession was learned 
	-- as primary and secondary profession had lover/higher skill level
    -- than primary or does not exist.
	_profApi:setFirstProf(ENCHANTING_LVL367_INDEX10)
	_profApi:setSecondProf(INSCRIPTION_LVL1_INDEX9)
	-- Hypnotist's Watch 25786 
	bagClass:setBagSlot(HYPNOTISTS_WATCH_TRINKET_1_1)
	self:assertEquals(DestroyusPro:isItemDisenchantable(
		              HYPNOTISTS_WATCH_TRINKET_1_1["i"],
					  HYPNOTISTS_WATCH_TRINKET_1_1["j"]),
	                  true, "Hypnotist's Watch 25786 Bug Test")
end

function ItemsTest.test_isItemIlvlForUpgradedItems(self)
	bagClass:setBagSlot(KALDELAR_SABATONS_3_2)
	self.itemApi:setItemPos(KALDELAR_SABATONS_3_2["i"], KALDELAR_SABATONS_3_2["j"])
	self:assertEquals(self.itemApi:getILvl(), 303, "Ilvl of the item is different")
	self:assertEquals(self.itemApi:getQuality(), 3, "Quality of the item is different")
end

function ItemsTest.test_professionNotExist(self)
	_profApi:setFirstProf(UNKNOW_PROFESSIONS)
	_profApi:setSecondProf(UNKNOW_PROFESSIONS)
	-- Hypnotist's Watch 25786  
	bagClass:setBagSlot(HYPNOTISTS_WATCH_TRINKET_1_1)
	self:assertEquals(DestroyusPro:isItemDisenchantable(
	                  HYPNOTISTS_WATCH_TRINKET_1_1["i"],
					  HYPNOTISTS_WATCH_TRINKET_1_1["j"]),
	                  false, "Hypnotist's Watch 25786 Bug Test")
end

function ItemsTest.test_isNotUniqueItem(self) 
	-- Monastic Ring of Stoicism  88638  UniqueId
	self:assertEquals(DestroyusPro:isNotUniqueItem(MONASTIC_RING_OF_STOICISM_1_1["itemId"],
                      DestroyusPro.itemsIdNotForDisenchant),
	                  false, "Monastic Ring of Stoicism  88638  UniqueId")
	self:assertEquals(DestroyusPro:isNotUniqueItem(SEASONED_HACKER_ONE_HAND_AXE_1_1["itemId"],
	                  DestroyusPro.itemsIdNotForDisenchant),
	                  false, "Item should be unqiue!") 
	self:assertEquals(DestroyusPro:isNotUniqueItem(SUNSPRING_VEST_1_1["itemId"],
	                  DestroyusPro.itemsIdNotForDisenchant),
	                  true, "Item should not be unique!")
end

function ItemsTest.test_isItemDisenchantableDifferentSettings(self)
	_profApi:setFirstProf(JEWELCRAFTING_LVL125_INDEX2)
	_profApi:setSecondProf(ENCHANTING_LVL421_INDEX3)
	
	-- Emergency Hatchet 69223 One Hand Axe
	local previousValue = _G["DestroyusProItemQualityValue"]["uncommon"]
	_G["DestroyusProItemQualityValue"]["uncommon"] = -1
	bagClass:setBagSlot(EMERGENCY_HATCHET_ONE_HAND_AXE_1_1)
	self:assertEquals(DestroyusPro:isItemDisenchantable(
					  EMERGENCY_HATCHET_ONE_HAND_AXE_1_1["i"],
	                  EMERGENCY_HATCHET_ONE_HAND_AXE_1_1["j"]),
	                  false, "Uncommon field is disabled. Should not disenchant")
	_G["DestroyusProItemQualityValue"]["uncommon"] = previousValue
	
	-- Shoulderplates of the Eternal 45320 Shoulder
	previousValue = _G["DestroyusProItemQualityValue"]["epic"]
	_G["DestroyusProItemQualityValue"]["epic"] = -1
	bagClass:setBagSlot(SHOULDERPLATES_OF_THE_ETERNAL_3_2)
	self:assertEquals(DestroyusPro:isItemDisenchantable(
		              SHOULDERPLATES_OF_THE_ETERNAL_3_2["i"],
					  SHOULDERPLATES_OF_THE_ETERNAL_3_2["j"]),	
	                  false, "Epic field is disabled. Should not disenchant")
					  
	-- Felsoul Vortex 147112 Artifact Weapon
	bagClass:setBagSlot(FELSOUL_VORTEX_1_1)
	self:assertEquals(DestroyusPro:isItemDisenchantable(
		              FELSOUL_VORTEX_1_1["i"],
					  FELSOUL_VORTEX_1_1["j"]),	
	                  false, "Epic field is disabled. Felsoul Vortex 147112 Artifact Weapon")
					  
	_G["DestroyusProItemQualityValue"]["epic"] = previousValue
	
	-- Glaive of the Fallen 141604 Warglaives
	previousValue = _G["DestroyusProItemQualityValue"]["rare"]
	_G["DestroyusProItemQualityValue"]["rare"] = -1
	bagClass:setBagSlot(GLAIVE_OF_THE_FALLEN_WARGLAIVES_1_1)
	self:assertEquals(DestroyusPro:isItemDisenchantable(
		              GLAIVE_OF_THE_FALLEN_WARGLAIVES_1_1["i"],
					  GLAIVE_OF_THE_FALLEN_WARGLAIVES_1_1["j"]),	
	                  false, "Rare field is disabled. Should not disenchant")
	_G["DestroyusProItemQualityValue"]["rare"] = previousValue

	-- Emergency Hatchet 69223 One Hand Axe
	bagClass:setBagSlot(EMERGENCY_HATCHET_ONE_HAND_AXE_1_1)
	self:assertEquals(DestroyusPro:isItemDisenchantable(
					  EMERGENCY_HATCHET_ONE_HAND_AXE_1_1["i"],
	                  EMERGENCY_HATCHET_ONE_HAND_AXE_1_1["j"]),
	                  true, "Uncommon field is enabled. Should  disenchant")
	
	-- Shoulderplates of the Eternal 45320 Shoulder
	bagClass:setBagSlot(SHOULDERPLATES_OF_THE_ETERNAL_3_2)
	self:assertEquals(DestroyusPro:isItemDisenchantable(
		              SHOULDERPLATES_OF_THE_ETERNAL_3_2["i"],
					  SHOULDERPLATES_OF_THE_ETERNAL_3_2["j"]),	
	                  true, "Epic field is enabled. Should  disenchant")
	
	-- Glaive of the Fallen 141604 Warglaives
	bagClass:setBagSlot(GLAIVE_OF_THE_FALLEN_WARGLAIVES_1_1)
	self:assertEquals(DestroyusPro:isItemDisenchantable(
		              GLAIVE_OF_THE_FALLEN_WARGLAIVES_1_1["i"],
					  GLAIVE_OF_THE_FALLEN_WARGLAIVES_1_1["j"]),	
	                  true, "Rare field is enabled. Should  disenchant")

	-- Felsoul Vortex 147112 Artifact Weapon
	bagClass:setBagSlot(FELSOUL_VORTEX_1_1)
	self:assertEquals(DestroyusPro:isItemDisenchantable(
		              FELSOUL_VORTEX_1_1["i"],
					  FELSOUL_VORTEX_1_1["j"]),	
	                  true, "Rare field is enabled. Felsoul Vortex 147112 Artifact Weapon")
end
